
# ###################################################################
# Import packages
# ###################################################################

import struct # for binary representation
import socket # for hostname identification
import string # for ascii selection
from datetime import datetime, timezone
from twisted.python import log
import serial
import os, csv
import sys

def send_command_ascii(ser,command,eol):
    #use printable here
    response = ''
    command = eol+command+eol
    if(ser.isOpen() == False):
        ser.open()
    sendtime = datetime.now(timezone.utc).replace(tzinfo=None)
    # encode to binary if python3
    if sys.version_info >= (3, 0):
        ser.write(command.encode('ascii'))
    else:
        ser.write(command)
    #ser.write(command)
    # skipping all empty lines
    while response == '':
        response = ser.readline()
    responsetime = datetime.now(timezone.utc).replace(tzinfo=None)
    # decode from binary if py3
    if sys.version_info >= (3, 0):
        response = response.decode('ascii')
    # return only ascii
    line = ''.join(filter(lambda x: x in string.printable, response))
    line = line.strip()
    return line, responsetime

def dataToCSV(outputdir, sensorid, filedate, asciidata, header):
                # Will be part of acquisitionsupport from MagPy 0.4.5
                #try:
                path = os.path.join(outputdir,sensorid)
                if not os.path.exists(path):
                    os.makedirs(path)
                savefile = os.path.join(path, sensorid+'_'+filedate+".asc")
                asciilist = asciidata.split(';')
                if not os.path.isfile(savefile):
                    with open(savefile, "wb") as csvfile:
                        writer = csv.writer(csvfile,delimiter=';')
                        writer.writerow(header)
                        writer.writerow(asciilist)
                else:
                    with open(savefile, "a") as csvfile:
                        writer = csv.writer(csvfile,delimiter=';')
                        writer.writerow(asciilist)
                #except:
                #log.err("datatoCSV: Error while saving file")        


def datetime2array(t):
    return [t.year,t.month,t.day,t.hour,t.minute,t.second,t.microsecond]

## Arduino active request protocol
## --------------------

class DisdroProtocol(object):
    """
    Protocol to read data from a Disdrometer

    Active protocol to periodically request data 
    Example sensor.cfg:
    LNM_0351_0001,S0,115200,8,1,N,active,None,60,1,Disdro,LNM,0351,0001,-,MeteoTower,NTP,environment,Thiess LNM Disdrometer
    """

    ## need a reference to our WS-MCU gateway factory to dispatch PubSub events
    ##
    def __init__(self, client, sensordict, confdict):
        # Specific commands for Disdrometer
        self.commands = [{'data':'11TR00005'}]

        #self.commands = [{'data':'01TR00002','c1':'01SH','c2':'01SL'}]
        self.hexcoding = False
        self.eol = '\r'


        self.client = client #self.wsMcuFactory = wsMcuFactory
        self.sensordict = sensordict
        self.confdict = confdict
        self.count = 0  ## counter for sending header information

        self.sensorid = ''
        self.sensorname = sensordict.get('name')
        self.baudrate=int(sensordict.get('baudrate'))
        self.port = confdict['serialport']+sensordict.get('port')
        self.parity=sensordict.get('parity')
        self.bytesize=sensordict.get('bytesize')
        self.stopbits=sensordict.get('stopbits')
        self.timeout=2 # should be rate depended

        self.hostname = socket.gethostname()
        self.printable = set(string.printable)
        self.datalst = []
        self.datacnt = 0
        rate = sensordict.get('rate','0')
        try:
            rate = int(rate)
        except:
            rate = 1
        if rate < 5:
            self.metacnt = 10 # send header at reduced rate
        else:
            self.metacnt = 1
        self.counter = {}

        # QOS
        self.qos=int(confdict.get('mqttqos',0))
        if not self.qos in [0,1,2]:
            self.qos = 0
        log.msg("  -> setting QOS:", self.qos)

        # switch on debug mode
        debugtest = confdict.get('debug')
        self.debug = False
        if debugtest == 'True':
            log.msg('DEBUG - {}: Debug mode activated.'.format(self.sensordict.get('protocol')))
            self.debug = True    # prints many test messages
        else:
            log.msg('  -> Debug mode = {}'.format(debugtest))

        # DISDROMETER specific
        self.sensorid = self.sensorname


    def processData(self, line, ntptime):
        """processing and storing data - requires sensorid and meta info
           data looks like (TR00005):

           The MARTAS code reads only some parameters and sends them in realtime. The full raw data
           is saved into a ascii file with the bufferdirectory
        """
        # currenttime = datetime.now(timezone.utc).replace(tzinfo=None)
        outdate = datetime.strftime(ntptime, "%Y-%m-%d")
        timestamp = datetime.strftime(ntptime, "%Y-%m-%d %H:%M:%S.%f")
        filename = outdate
        header = ''
        datearray = datetime2array(ntptime)
        packcode = '6hL'
        multiplier = []
        pc = []
        key = '[x,y,z,f,t1,t2,var1,var2,var3,var4,dx,dy,dz,str1]'
        ele = '[rainfall,visibility,reflectivity,P_tot,T,T_el,I_tot,I_fluid,I_solid,d(hail),P_slow,P_fast,P_small,SYNOP-4680-code]'
        unit = '[mm,m,dBZ,None,degC,degC,None,None,None,mm,None,None,None,None]'  ### check database
        #print ("Processing line for {}: {}".format(sensorid, line))
        data = line.split(';')

        def addpara(para, mu, c='l',pc=pc, multplier=multiplier):
            datearray.append(int(para*mu))
            pc.append(c)
            multiplier.append(mu)
            return pc, multiplier

        if len(data) > 10:
            if self.debug:
                print ("Received data - amount of elements: {}".format(len(data)))
            try:
                # Extract data
                #sensor = 'LNM'
                serialnum = data[1] # I guess that this is the serial connectors ID...
                cumulativerain = float(data[15]) 	# x
                pc, multiplier = addpara(cumulativerain,100,c='l')
                if cumulativerain > 9000:
                    log.msg('SerialCall - writeDisdro: Resetting percipitation counter')
                    self.commands = [{'setconfmod':'11KY00001'},{'reset':'11RA00001'},{'setworkmode':'11KY00000'}]
                visibility = int(data[16])		# y
                pc, multiplier = addpara(visibility,1,c='l')
                reflectivity = float(data[17])	 	# z
                pc, multiplier = addpara(reflectivity,100)
                Ptotal= int(data[49])			# f
                pc, multiplier = addpara(Ptotal,1)
                outsidetemp = float(data[44])		# t1
                pc, multiplier = addpara(outsidetemp,100)
                insidetemp = float(data[36])	 	# t2
                pc, multiplier = addpara(insidetemp,100)
                intall = float(data[12]	)	 	# var1
                pc, multiplier = addpara(intall,100)
                intfluid = float(data[13])	 	# var2
                pc, multiplier = addpara(intfluid,100)
                intsolid = float(data[14])	 	# var3
                pc, multiplier = addpara(intsolid,100)
                quality = int(data[18])
                haildiameter = float(data[19])		# var4
                pc, multiplier = addpara(haildiameter,100)
                lasertemp = float(data[37])
                lasercurrent = data[38]
                Pslow = int(data[51])		 	# dx
                pc, multiplier = addpara(Pslow,1)
                Pfast= int(data[53])		 	# dy
                pc, multiplier = addpara(Pfast,1)
                Psmall= int(data[55])		 	# dz
                pc, multiplier = addpara(Psmall,1)
                synop = data[6]                         # str1
                pc, multiplier = addpara(synop,1,c='s')
                revision = '0001' # Software version 2.42
                self.sensorid = self.sensorname + '_' + serialnum + '_' + revision
                data = ','.join(list(map(str,datearray)))
                packcode = packcode+''.join(list(pc))
                header = "# MagPyBin {} {} {} {} {} {} {}".format (self.sensorid, key, ele, unit, '['+','.join(list(map(str,multiplier)))+']', packcode, struct.calcsize('<'+packcode))
                if self.debug:
                    print ("Header", header)
            except:
                log.err('SerialCall - writeDisdro: Could not assign data values')
                data = ''


            if not self.confdict.get('bufferdirectory','') == '':
                timestr = timestamp.replace(' ',';')
                asciiline = ''.join([i for i in line if ord(i) < 128])
                asciidata = timestr + ';' + asciiline.strip('\x03').strip('\x02')
                fheader = '# LNM - Telegram5 plus NTP date and time at position 0 and 1'
                try:
                    dataToCSV(self.confdict.get('bufferdirectory'),self.sensorid, filename, asciidata, [fheader])
                except:
                    log.msg("Writing data failed")
                #if self.debug:
                #    print ("  -> writing data successfull: {}".format(self.sensorid))
        else:
            data = ''

        return data, header


    def sendRequest(self):

        success = True
        # connect to serial
        ser = serial.Serial(self.port, baudrate=int(self.baudrate), parity=self.parity, bytesize=int(self.bytesize), stopbits=int(self.stopbits), timeout=int(self.timeout))
        for commdict in self.commands:
            for item in sorted(commdict):
                comm = commdict.get(item)
                if self.debug:
                    print ("sending item {} with command {}".format(item, comm))
                # eventually more frequently ask for 'data'
                answer, actime = send_command_ascii(ser,comm,self.eol)
                answerok = True
                if item in ['setworkmode','setconfmode','reset']:
                    answerok = False
                if item == 'setworkmode':
                    log.msg('SerialCall: Continuing normal acquisition')
                    self.commands = [{'data':'11TR00005'}]
                # disconnect from serial
                ser.close()
                #if self.debug:
                #    print ("got answer: {}".format(answer))
                # check answer
                if answerok:
                    data, head = self.processData(answer, actime)
                    # send data via mqtt
                    if not data == '':
                        self.sendmqtt(self.sensorid,data,head)

                # ##########################################################
                # answer is a string block with eventually multiple lines
                # apply the old linereceived method to each line of the answer
                # thats it ...

                #answer = ''.join(filter(lambda x: x in string.printable, answer))
                """
                if not success and self.errorcnt < 5 and not item=='reset':
                    self.errorcnt = self.errorcnt + 1
                    log.msg('SerialCall: Could not interprete response of system when sending %s' % item) 
                elif not success and self.errorcnt == 5:
                    try:
                        check_call(['/etc/init.d/martas', 'restart'])
                    except subprocess.CalledProcessError:
                        log.msg('SerialCall: check_call didnt work')
                        pass # handle errors in the called executable
                    except:
                        log.msg('SerialCall: check call problem')
                        pass # executable not found
                    #os.system("/etc/init.d/martas restart")
                    log.msg('SerialCall: Restarted martas process')
                """
        # disconnect from serial
        #ser.close()

    def sendmqtt(self,sensorid,data,head):
        """
            call this method after processing data
        """

        ok = True
        if ok:

            topic = self.confdict.get('station') + '/' + sensorid

            senddata = False
            try:
                coll = int(self.sensordict.get('stack'))
            except:
                coll = 0

            if coll > 1:
                self.metacnt = 1 # send meta data with every block
                if self.datacnt < coll:
                    self.datalst.append(data)
                    self.datacnt += 1
                else:
                    senddata = True
                    data = ';'.join(self.datalst)
                    self.datalst = []
                    self.datacnt = 0
            else:
                senddata = True

            if senddata:
                if self.debug:
                    print ("publishing...")
                self.client.publish(topic+"/data", data, qos=self.qos)
                # If multiple sensors are connected, self.count needs to be a dict (with topic)
                # Initialize counter for each topic
                if self.counter.get(sensorid,'') == '':
                    self.counter[sensorid] = 0
                cnt = self.counter.get(sensorid)

                if cnt == 0:
                    ## 'Add' is a string containing dict info like: 
                    ## SensorID:ENV05_2_0001,StationID:wic, PierID:xxx,SensorGroup:environment,... 
                    add = "SensorID:{},StationID:{},DataPier:{},SensorModule:{},SensorGroup:{},SensorDescription:{},DataTimeProtocol:{}".format( sensorid,self.confdict.get('station','').strip(),self.sensordict.get('pierid','').strip(),self.sensordict.get('protocol','').strip(),self.sensordict.get('sensorgroup','').strip(),self.sensordict.get('sensordesc','').strip(),self.sensordict.get('ptime','').strip() )
                    self.client.publish(topic+"/dict", add, qos=self.qos)
                    self.client.publish(topic+"/meta", head, qos=self.qos)

                cnt += 1
                if cnt >= self.metacnt:
                    cnt = 0
                # update counter in dict 
                self.counter[sensorid] = cnt




#!/usr/bin/env python
# coding=utf-8

"""
Speedtest:

Perform a speedtest based on speedtest-cli
(https://www.speedtest.net/de/apps/cli)

sudo apt install speedtest-cli


################################

add crontab to regularly run monitor (root)
sudo crontab -e
*/5  *  *  *  *  /usr/bin/python3 /path/to/speedtest.py -c /path/to/conf.cfg -n speed_starlink01_0001  > /dev/NULL 2&>1
"""

# Define packages to be used (local refers to test environment)
# ------------------------------------------------------------
import os, sys, getopt
from datetime import datetime
from martas.version import __version__

import socket

import subprocess
import dateutil.parser
import csv


def getspeed(server=None, debug=False):

    # TODO test what happens if network is not available
    call = ["speedtest",'--csv']
    if debug:
        print ("Executing script ...")
    proc=subprocess.Popen(call, stdout=subprocess.PIPE)
    output = proc.stdout.read()
    outputstr = output.decode('utf-8')
    if debug:
        print (outputstr)
    result = outputstr.strip().split(',')
    print (result)
    return result
    
def write_basic_ascii(result,path='/srv/mqtt/',speedtestname='speed_666_0001', debug = False):
    success = True
    # Header
    header = ['DT_datatime','N_latency[ms]','N_download[Mbit/s]','N_upload[Mbit/s]','N_serverdistance[km]','S_sever','S_location']
    # Filename
    # makes use of datetime
    dat = dateutil.parser.isoparse(result[3])
    print (dat)
    datum = datetime.strftime(dat,"%Y-%m-%d")
    filename = "{}_{}.asc".format(speedtestname,datum)    
    print (filename)
    ### Create a simple ascii csv output
    # header contains DT=Datetime, N=Numeric, S=String
    #DT_datatime, N_latency[ms], N_download[Mbyte/s], N_upload[Mbyte/s], N_serverdistance[km], S_sever, S_location
    #
    outputlist = [result[3],float(result[5]),float(result[6])/1000000.,float(result[7])/1000000.,float(result[4]),result[1],result[2]]
    print (outputlist)
    # Write CSV
    filepath = os.path.join(path,speedtestname,filename)
    dirpath = os.path.join(path,speedtestname)
    if not debug:
        # 1. create directory if not existing
        if not os.path.exists(dirpath):
            os.makedirs(dirpath)
        # 2. write header if file not existing
        if not os.path.isfile(filepath):
            print ("creating new file")
            with open(filepath, 'w', newline='') as myfile:
                wr = csv.writer(myfile)
                wr.writerow(header)
        # 3. append data if file exists
        if os.path.exists(filepath):
            with open(filepath, 'a', newline='') as myfile:
                wr = csv.writer(myfile)
                wr.writerow(outputlist)
    else:
        print ("debug selected - skipping write")

    return success
    

def main(argv):
    version = __version__
    statusmsg = {}
    configpath = ''
    hostname = socket.gethostname().upper()
    debug = False
    travistestrun = False
    speedtestname = 'speed_starlink01_0001'
    outpath = '/srv/mqtt/'
    #testst = DataStream()

    try:
        opts, args = getopt.getopt(argv,"hc:n:p:vDT",["config=","name=","path=",])
    except getopt.GetoptError:
        print ('speedtest.py -c <config> -n <name> -p <path> -v <version>')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print ('------------------------------------------------------------')
            print ('Description:')
            print ('-- speedtest.py to monitor internet bandwidth  --')
            print ('------------------------------------------------------------')
            print ('-------------------------------------')
            print ('Usage:')
            print ('speedtest.py -c <config> -n <name>  -p <path>')
            print ('-------------------------------------')
            print ('Options:')
            print ('-c (coming soon) : path to a configuration file')
            print ('-n            : virtual sensor name; default: speed_starlink01_0001 ')
            print ('-p            : path for file storage; default: /srv/mqtt/ ')
            print ('-v            : print the current version of speedtest.py')
            print ('-------------------------------------')
            print ('Application:')
            print ('python3 speedtest.py -c /etc/martas/appconf/monitor.cfg')
            sys.exit()
        elif opt in ("-c", "--config"):
            configpath = arg
        elif opt in ("-n", "--name"):
            speedtestname = arg
        elif opt in ("-p", "--path"):
            outpath = arg
        elif opt == "-v":
            print ("speed.py version: {}".format(version))
        elif opt in ("-D", "--debug"):
            debug = True
        elif opt in ("-T", "--test"):
            travistestrun = True

    result = getspeed(debug=debug)
    write_basic_ascii(result,path=outpath,speedtestname=speedtestname, debug=debug)

if __name__ == "__main__":
   main(sys.argv[1:])






# Author: Johan Hanssen Seferidis
# License: MIT

import re
import sys
import struct
from base64 import b64encode
from hashlib import sha1
import logging

if sys.version_info[0] < 3:
    from SocketServer import ThreadingMixIn, TCPServer, StreamRequestHandler
else:
    from socketserver import ThreadingMixIn, TCPServer, StreamRequestHandler

logger = logging.getLogger(__name__)
logging.basicConfig()

'''
+-+-+-+-+-------+-+-------------+-------------------------------+
 0                   1                   2                   3
 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+-+-+-+-+-------+-+-------------+-------------------------------+
|F|R|R|R| opcode|M| Payload len |    Extended payload length    |
|I|S|S|S|  (4)  |A|     (7)     |             (16/64)           |
|N|V|V|V|       |S|             |   (if payload len==126/127)   |
| |1|2|3|       |K|             |                               |
+-+-+-+-+-------+-+-------------+ - - - - - - - - - - - - - - - +
|     Extended payload length continued, if payload len == 127  |
+ - - - - - - - - - - - - - - - +-------------------------------+
|                     Payload Data continued ...                |
+---------------------------------------------------------------+
'''

FIN    = 0x80
OPCODE = 0x0f
MASKED = 0x80
PAYLOAD_LEN = 0x7f
PAYLOAD_LEN_EXT16 = 0x7e
PAYLOAD_LEN_EXT64 = 0x7f

OPCODE_CONTINUATION = 0x0
OPCODE_TEXT         = 0x1
OPCODE_BINARY       = 0x2
OPCODE_CLOSE_CONN   = 0x8
OPCODE_PING         = 0x9
OPCODE_PONG         = 0xA


# -------------------------------- API ---------------------------------

class API():

    def run_forever(self):
        try:
            logger.info("Listening on port %d for clients.." % self.port)
            self.serve_forever()
        except KeyboardInterrupt:
            self.server_close()
            logger.info("Server terminated.")
        except Exception as e:
            logger.error(str(e), exc_info=True)
            exit(1)

    def new_client(self, client, server):
        pass

    def client_left(self, client, server):
        pass

    def message_received(self, client, server, message):
        pass

    def set_fn_new_client(self, fn):
        self.new_client = fn

    def set_fn_client_left(self, fn):
        self.client_left = fn

    def set_fn_message_received(self, fn):
        self.message_received = fn

    def send_message(self, client, msg):
        self._unicast_(client, msg)

    def send_message_to_all(self, msg):
        self._multicast_(msg)


# ------------------------- Implementation -----------------------------

class WebsocketServer(ThreadingMixIn, TCPServer, API):
    """
        A websocket server waiting for clients to connect.

    Args:
        port(int): Port to bind to
        host(str): Hostname or IP to listen for connections. By default 127.0.0.1
            is being used. To accept connections from any client, you should use
            0.0.0.0.
        loglevel: Logging level from logging module to use for logging. By default
            warnings and errors are being logged.

    Properties:
        clients(list): A list of connected clients. A client is a dictionary
            like below.
                {
                 'id'      : id,
                 'handler' : handler,
                 'address' : (addr, port)
                }
    """

    allow_reuse_address = True
    daemon_threads = True  # comment to keep threads alive until finished

    clients = []
    id_counter = 0

    def __init__(self, port, host='127.0.0.1', loglevel=logging.WARNING):
        logger.setLevel(loglevel)
        self.port = port
        TCPServer.allow_reuse_address = True
        TCPServer.__init__(self, (host, port), WebSocketHandler)

    def _message_received_(self, handler, msg):
        self.message_received(self.handler_to_client(handler), self, msg)

    def _ping_received_(self, handler, msg):
        handler.send_pong(msg)

    def _pong_received_(self, handler, msg):
        pass

    def _new_client_(self, handler):
        self.id_counter += 1
        client = {
            'id': self.id_counter,
            'handler': handler,
            'address': handler.client_address
        }
        self.clients.append(client)
        self.new_client(client, self)

    def _client_left_(self, handler):
        client = self.handler_to_client(handler)
        self.client_left(client, self)
        if client in self.clients:
            self.clients.remove(client)

    def _unicast_(self, to_client, msg):
        to_client['handler'].send_message(msg)

    def _multicast_(self, msg):
        for client in self.clients:
            self._unicast_(client, msg)

    def handler_to_client(self, handler):
        for client in self.clients:
            if client['handler'] == handler:
                return client


class WebSocketHandler(StreamRequestHandler):

    def __init__(self, socket, addr, server):
        self.server = server
        StreamRequestHandler.__init__(self, socket, addr, server)

    def setup(self):
        StreamRequestHandler.setup(self)
        self.keep_alive = True
        self.handshake_done = False
        self.valid_client = False

    def handle(self):
        while self.keep_alive:
            if not self.handshake_done:
                self.handshake()
            elif self.valid_client:
                self.read_next_message()

    def read_bytes(self, num):
        # python3 gives ordinal of byte directly
        bytes = self.rfile.read(num)
        if sys.version_info[0] < 3:
            return map(ord, bytes)
        else:
            return bytes

    def read_next_message(self):
        try:
            b1, b2 = self.read_bytes(2)
        except ValueError as e:
            b1, b2 = 0, 0

        fin    = b1 & FIN
        opcode = b1 & OPCODE
        masked = b2 & MASKED
        payload_length = b2 & PAYLOAD_LEN

        if not b1:
            logger.info("Client closed connection.")
            self.keep_alive = 0
            return
        if opcode == OPCODE_CLOSE_CONN:
            logger.info("Client asked to close connection.")
            self.keep_alive = 0
            return
        if not masked:
            logger.warn("Client must always be masked.")
            self.keep_alive = 0
            return
        if opcode == OPCODE_CONTINUATION:
            logger.warn("Continuation frames are not supported.")
            return
        elif opcode == OPCODE_BINARY:
            logger.warn("Binary frames are not supported.")
            return
        elif opcode == OPCODE_TEXT:
            opcode_handler = self.server._message_received_
        elif opcode == OPCODE_PING:
            opcode_handler = self.server._ping_received_
        elif opcode == OPCODE_PONG:
            opcode_handler = self.server._pong_received_
        else:
            logger.warn("Unknown opcode %#x." + opcode)
            self.keep_alive = 0
            return

        if payload_length == 126:
            payload_length = struct.unpack(">H", self.rfile.read(2))[0]
        elif payload_length == 127:
            payload_length = struct.unpack(">Q", self.rfile.read(8))[0]

        masks = self.read_bytes(4)
        decoded = ""
        for char in self.read_bytes(payload_length):
            char ^= masks[len(decoded) % 4]
            decoded += chr(char)
        opcode_handler(self, decoded)

    def send_message(self, message):
        self.send_text(message)

    def send_pong(self, message):
        self.send_text(message, OPCODE_PONG)

    def send_text(self, message, opcode=OPCODE_TEXT):
        """
        Important: Fragmented(=continuation) messages are not supported since
        their usage cases are limited - when we don't know the payload length.
        """

        # Validate message
        if isinstance(message, bytes):
            message = try_decode_UTF8(message)  # this is slower but ensures we have UTF-8
            if not message:
                logger.warning("Can\'t send message, message is not valid UTF-8")
                return False
        elif sys.version_info < (3,0) and (isinstance(message, str) or isinstance(message, unicode)):
            pass
        elif isinstance(message, str):
            pass
        else:
            logger.warning('Can\'t send message, message has to be a string or bytes. Given type is %s' % type(message))
            return False

        header  = bytearray()
        payload = encode_to_UTF8(message)
        payload_length = len(payload)

        # Normal payload
        if payload_length <= 125:
            header.append(FIN | opcode)
            header.append(payload_length)

        # Extended payload
        elif payload_length >= 126 and payload_length <= 65535:
            header.append(FIN | opcode)
            header.append(PAYLOAD_LEN_EXT16)
            header.extend(struct.pack(">H", payload_length))

        # Huge extended payload
        elif payload_length < 18446744073709551616:
            header.append(FIN | opcode)
            header.append(PAYLOAD_LEN_EXT64)
            header.extend(struct.pack(">Q", payload_length))

        else:
            raise Exception("Message is too big. Consider breaking it into chunks.")
            return

        self.request.send(header + payload)

    def handshake(self):
        message = self.request.recv(1024).decode().strip()
        upgrade = re.search('\nupgrade[\s]*:[\s]*websocket', message.lower())
        if not upgrade:
            self.keep_alive = False
            return
        key = re.search('\n[sS]ec-[wW]eb[sS]ocket-[kK]ey[\s]*:[\s]*(.*)\r\n', message)
        if key:
            key = key.group(1)
        else:
            logger.warning("Client tried to connect but was missing a key")
            self.keep_alive = False
            return
        response = self.make_handshake_response(key)
        self.handshake_done = self.request.send(response.encode())
        self.valid_client = True
        self.server._new_client_(self)

    def make_handshake_response(self, key):
        return \
          'HTTP/1.1 101 Switching Protocols\r\n'\
          'Upgrade: websocket\r\n'              \
          'Connection: Upgrade\r\n'             \
          'Sec-WebSocket-Accept: %s\r\n'        \
          '\r\n' % self.calculate_response_key(key)

    def calculate_response_key(self, key):
        GUID = '258EAFA5-E914-47DA-95CA-C5AB0DC85B11'
        hash = sha1(key.encode() + GUID.encode())
        response_key = b64encode(hash.digest()).strip()
        return response_key.decode('ASCII')

    def finish(self):
        self.server._client_left_(self)


def encode_to_UTF8(data):
    try:
        return data.encode('UTF-8')
    except UnicodeEncodeError as e:
        logger.error("Could not encode data to UTF-8 -- %s" % e)
        return False
    except Exception as e:
        raise(e)
        return False


def try_decode_UTF8(data):
    try:
        return data.decode('utf-8')
    except UnicodeDecodeError:
        return False
    except Exception as e:
        raise(e)



#!/usr/bin/env python3
# coding=utf-8

"""
MARTAS - MagPy's automatic real time acquisition system

martas_init will create folders, setup all configuration files, eventually install dependencies

"""

import shutil
import getopt
import sys
import os
from magpy.opt import cred
from pathlib import Path
from crontab import CronTab
import martas
from martas.version import __version__
import getpass

def main(argv):
    debug = False
    dir = ".martas"
    redo = False
    update = False
    minimalupdate = False

    try:
        opts, args = getopt.getopt(argv,"hd:ruUD",["path=","redo=","update=","UPADTE=","debug=",])
    except getopt.GetoptError:
        print ('martas_init')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print ('-------------------------------------')
            print ('Description:')
            print ('-- martas_init will initialize imbot configuration --')
            print ('-----------------------------------------------------------------')
            print ('martas_init will perform the following tasks:')
            print ('- program files will always be stored within your home directory')
            print ('- default directory is .martas, change that using -d option')
            print ('- will create a ~/.martas directory')
            print ('- copy skeleton configuration files to .martas/conf/')
            print ('- copy bash scripts to .martas/scripts/')
            print ('- copy python applications to .martas/app/')
            print ('')
            print ('Options:')
            print ('-d, --directory : define the main configuration directory')
            print ('                : i.e. -d MARTAS will store everything within /hone/user/MARTAS')
            print ('-u, --update : will update applications (app, doc, web) folder with new version.')
            print ('             : Then you will asked to input your new job or update an exsiting.')
            print ('             : A backup is advisable before updating')
            print ('-U, --UPDATE : will ONLY update applications (app, doc, web) folder with new version')
            print ('-r, --redo   : replace all already existing configuration files.')
            print ('             : ATTENTION: redo will delete all previous configurations')
            print ('-------------------------------------')
            print ('Application:')
            print ('-------------------------------------')
            print ('python3 martas_init.py')
            sys.exit()
        elif opt in ("-d", "--directory"):
            dir = arg
        elif opt in ("-r", "--redo"):
            redo = True
        elif opt in ("-u", "--update"):
            update = True
        elif opt in ("-U", "--UPDATE"):
            minimalupdate = True
        elif opt in ("-D", "--debug"):
            debug = True

    # get home directory of current user
    homedir = os.getenv("HOME")

    envact = ""
    # get the environment
    try:
        # Scan for conda environment
        env = os.environ["CONDA_PREFIX"]
        # will cause a KEY ERROR if not existing
        envname = os.path.basename(os.path.normpath(env))
        if env.endswith("envs/{}".format(envname)):
            envact = "conda activate {}".format(envname)
        else:
            envact = "conda activate base"
    except:
        # No conda environment - test for virtualenv
        try:
            env = os.environ["VIRTUAL_ENV"]
            envact = "source {}".format(os.path.join(env,"bin","activate"))
        except:
            pass

    # get the martas package path
    file_path = os.path.dirname(martas.__file__)
    if not debug:
        os.makedirs(os.path.join(homedir,dir), exist_ok=True)
        # create sudirs
        os.makedirs(os.path.join(homedir,dir,"log"), exist_ok=True)
    #
    # copy files into subdirs
    if update or minimalupdate:
        print ("Updating app, doc and web to MARTAS version {}".format(__version__))
        shutil.rmtree(os.path.join(homedir, dir, "app"),ignore_errors=True)
        shutil.rmtree(os.path.join(homedir, dir, "doc"),ignore_errors=True)
        shutil.rmtree(os.path.join(homedir, dir, "web"),ignore_errors=True)
    if redo:
        shutil.rmtree(os.path.join(homedir, dir, "app"),ignore_errors=True)
        shutil.rmtree(os.path.join(homedir, dir, "conf"),ignore_errors=True)
        shutil.rmtree(os.path.join(homedir, dir, "doc"),ignore_errors=True)
        shutil.rmtree(os.path.join(homedir, dir, "init"),ignore_errors=True)
        shutil.rmtree(os.path.join(homedir, dir, "logrotate"),ignore_errors=True)
        shutil.rmtree(os.path.join(homedir, dir, "web"),ignore_errors=True)
    if not os.path.isdir(os.path.join(homedir,dir,"conf")):
        shutil.copytree(os.path.join(file_path, "conf"), os.path.join(homedir, dir, "conf"))
    if not os.path.isdir(os.path.join(homedir,dir,"app")):
        shutil.copytree(os.path.join(file_path, "app"), os.path.join(homedir, dir, "app"))
    if not os.path.isdir(os.path.join(homedir,dir,"doc")):
        shutil.copytree(os.path.join(file_path, "doc"), os.path.join(homedir, dir, "doc"))
    if not os.path.isdir(os.path.join(homedir,dir,"init")):
        shutil.copytree(os.path.join(file_path, "init"), os.path.join(homedir, dir, "init"))
    if not os.path.isdir(os.path.join(homedir,dir,"logrotate")):
        shutil.copytree(os.path.join(file_path, "logrotate"), os.path.join(homedir, dir, "logrotate"))
    if not os.path.isdir(os.path.join(homedir,dir,"scripts")):
        shutil.copytree(os.path.join(file_path, "scripts"), os.path.join(homedir, dir, "scripts"))
    if not os.path.isdir(os.path.join(homedir,dir,"web")):
        shutil.copytree(os.path.join(file_path, "web"), os.path.join(homedir, dir, "web"))
    #shutil.copyfile(os.path.join(file_path, "collector.py"), os.path.join(homedir, dir, "collector.py"))
    #shutil.copyfile(os.path.join(file_path, "acquisition.py"), os.path.join(homedir, dir, "acquisition.py"))
    #
    confpath = "/tmp"
    initpath = "/tmp"
    logpath = "/tmp"
    malogpath = "/tmp/martas.log"
    jobname = "martas"
    marcosjob = "marcos"
    initjob = "MARTAS"
    stationname = "WIC"
    mqttbroker = "localhost"
    mqttport = "1883"
    mqttqos = "1"
    mqttcred = ""
    backuppath = os.path.join(homedir, "backups")
    bufferpath = os.path.join(homedir, "MARTAS", "mqtt")
    archivepath = os.path.join(homedir, "MARCOS", "archive") # i.e. /home/USER/MARCOS/archive
    archivelog = "/tmp/archivestatus.log"
    monitorlog = "/tmp/monitorstatus.log"
    thresholdlog = "/tmp/thresholdstatus.log"
    thresholdsource = "file"
    destination = "stdout"
    filepath = "/tmp"
    databasecredentials = "mydb"
    payloadformat = "martas"
    noti = "telegram"
    mailcred = "mymailcred"
    mainpath = os.path.join(homedir,dir) # i.e. /home/USER/.martas
    pskcredentials = ""
    mqttcert = ""
    addtelegrambot = False


    if minimalupdate:
        print (" ------------------------------------------- ")
        print (" application, web and documentfolder have been updated")
        print (" ------------------------------------------- ")
        sys.exit(0)

    cronlist = []
    print (" ------------------------------------------- ")
    print ("""               You started the MARTAS initialization routine.
               Please provide some additional information so
               that MARTAS or MARCOS can be set up correctly.
               BEFORE CONTINUING:
               - MARTAS/MARCOS: make sure you have e-mail/telegram credentials
               - MARTAS/MARCOS: make sure you have MQTT credentials
               - MARTAS: accessible buffer directory
               - MARCOS: archiving requires accessible directory path
               - MARCOS: if you want to use a MySQL/Maria database
                         * install MariaDB and create an empty DB (see manual)
                         * credentials
                         * percona installed (apt install percona)
               - MARTAS: sensors need to be setup manually after init
               PLEASE NOTE: After finishing the setup you
               can modify all configurations manually anytime.
               Please consult the manual. """)

    print (" ------------------------------------------- ")
    print (" Please insert a path for the main configuration files:")
    print (" (press return for accepting the default: {})".format(os.path.join(homedir,dir,"conf")))
    newconfpath = input()
    if newconfpath:
        if not os.path.isdir(newconfpath):
            print ("  ! the selected directory is not yet existing - aborting")
            sys.exit()
        if not os.access(newconfpath, os.W_OK):
            print (" ! you don't have write access to the specific directory - aborting")
            sys.exit()
        confpath = newconfpath
        shutil.copytree(os.path.join(homedir, dir, "conf"), confpath)
    else:
        confpath = os.path.join(homedir,dir,"conf")
    shutil.copyfile(os.path.join(confpath, "archive.cfg"), os.path.join(confpath, "archive.bak"))
    shutil.copyfile(os.path.join(confpath, "basevalue.cfg"), os.path.join(confpath, "basevalue.bak"))
    if os.path.exists(os.path.join(confpath, "download-source.cfg")):
        shutil.copyfile(os.path.join(confpath, "download-source.cfg"), os.path.join(confpath, "download-source.bak"))
    if os.path.exists(os.path.join(confpath, "fileuploadjobs.json")):
        shutil.copyfile(os.path.join(confpath, "fileuploadjobs.json"), os.path.join(confpath, "fileuploadjobs.bak"))
    shutil.copyfile(os.path.join(confpath, "filter.cfg"), os.path.join(confpath, "filter.bak"))
    shutil.copyfile(os.path.join(confpath, "gamma.cfg"), os.path.join(confpath, "gamma.bak"))
    shutil.copyfile(os.path.join(confpath, "mail.cfg"), os.path.join(confpath, "mail.bak"))
    shutil.copyfile(os.path.join(confpath, "monitor.cfg"), os.path.join(confpath, "monitor.bak"))
    shutil.copyfile(os.path.join(confpath, "martas.cfg"), os.path.join(confpath, "martas.bak"))
    shutil.copyfile(os.path.join(confpath, "marcos.cfg"), os.path.join(confpath, "marcos.bak"))
    shutil.copyfile(os.path.join(confpath, "obsdaq.cfg"), os.path.join(confpath, "obsdaq.bak"))
    shutil.copyfile(os.path.join(confpath, "sensors.cfg"), os.path.join(confpath, "sensors.bak"))
    shutil.copyfile(os.path.join(confpath, "telegram.cfg"), os.path.join(confpath, "telegram.bak"))
    shutil.copyfile(os.path.join(confpath, "threshold.cfg"), os.path.join(confpath, "threshold.bak"))
    shutil.copyfile(os.path.join(confpath, "telegrambot.cfg"), os.path.join(confpath, "telegrambot.bak"))
    shutil.copyfile(os.path.join(confpath, "web.cfg"), os.path.join(confpath, "web.bak"))
    shutil.copyfile(os.path.join(homedir, dir, "scripts", "cleanup.sh"), os.path.join(homedir, dir, "scripts", "cleanup.bak"))

    print (" ------------------------------------------- ")
    print (" Please insert a path for log files:")
    print (" (press return for accepting default: {})".format(os.path.join(homedir,dir,"log")))
    newlogpath = input()
    if newlogpath:
        if not os.path.isdir(newlogpath):
            print ("  ! the selected directory is not yet existing - aborting")
            sys.exit()
        if not os.access(newlogpath, os.W_OK):
            print (" ! you don't have write access to the specific directory - aborting")
            sys.exit()
        logpath = newlogpath
        shutil.copytree(os.path.join(homedir, dir, "log"), logpath)
    else:
        logpath = os.path.join(homedir,dir,"log")

    print (" ------------------------------------------- ")
    print (" Please insert a station ID (i.e. the three letter observatory code, like WIC)")
    print (" (please note: station code is converted to lower case for martas mqtt topics)")
    print (" (press return for accepting default: {})".format(stationname))
    newstationname = input()
    if newstationname:
        stationname = ''.join(filter(str.isalnum, newstationname))
    print (" -> Station ID: {}".format(stationname))

    print (" ------------------------------------------- ")
    print (" Please insert a destination path for regular backups (default is ~/backups)")
    print (' (please insert "none" if you do not want regular backups of the martas system)')
    print (" (press return for accepting default: {})".format(os.path.join(homedir,"backups")))
    newbackuppath = input()
    if newstationname and not newstationname in ["NONE","none","no","n","NO","None","N"]:
        backupath = newbackuppath
    elif newstationname in ["NONE","none","no","n","NO","None","N"]:
        backuppath = ""
    print (" -> Station ID: {}".format(stationname))

    print (" ------------------------------------------- ")
    print (" Please insert the address of the MQTT broker:")
    print ("  MARTAS: the broker on which you publish data")
    print ("  MARCOS: the broker to which you subscribe")
    print (" (press return for accepting default: {})".format(mqttbroker))
    newmqttbroker = input()
    if newmqttbroker:
        mqttbroker = newmqttbroker
    print (" -> MQTT broker address: {}".format(mqttbroker))

    print(" ------------------------------------------- ")
    print(" Port of MQTT broker (default = {}):".format(mqttport))
    newmqttport = input()
    if newmqttport:
        mqttport = newmqttport
    if int(mqttport) >= 8883:
        print(" MQTT security based on TLS:")
        print(" please choose: (1) TLS-PSK encryption, (2) TLS on certificate basis")
        print(" (1) EXPERIMENTAL ---- requires an available PSK identity and password - AND - pip install sslpsk2")
        print(" (2) VERIFIED ---- most cases, also to be used for INTERMAGNET MQTT service")
        secsel = input()
        if secsel in ["1"]:
            print("  TLS-PSK: please insert a credential shortcut for a valid psk identity.")
            print("           Needs to be different to user credentials.")
            newpskcred = input()
            if newpskcred:
                # check whether existing
                pskcredentials = newpskcred
                # if not create
            val = cred.lc(pskcredentials, "user")
            if not val:
                print("  ---- ")
                print("  PSK Shortcut not yet existing - creating it: ")
                print("  Insert your PSK identity:")
                pskuser = input()
                print("  Insert PSK:")
                pskpwd = getpass.getpass()
                cred.cc("transfer", pskcredentials, user=pskuser, passwd=pskpwd, address=mqttbroker, port=int(mqttport))
            print(" -> MQTT psk identifier: {}".format(pskcredentials))
        elif secsel in ["2"]:
            print("  TLS with certificate: please insert path to ca.cert; leave empty for IM MQTT service or if you do not know what I am talking about")
            newmqttcert = input()
            if newmqttcert:
                if not os.path.isfile(newmqttcert):
                    print("  ! the selected ca.cert file is not existing")
                    print("  ! please update martas.cfg afterwards")
                mqttcert = newmqttcert
            print(" -> MQTT certificate: {}".format(mqttcert))

    print(" -> MQTT broker port: {}".format(mqttport))

    print (" ------------------------------------------- ")
    print (" MQTT transmission using quality of service (QOS): (default=1)".format(mqttqos))
    print ( " (can be 0,1,2 )")
    newmqttqos = input()
    if newmqttqos:
        if not newmqttqos in ["0","1","2"]:
            newmqttqos = "1"
        mqttqos = newmqttqos
    print (" -> MQTT QOS: {}".format(mqttqos))

    print (" ------------------------------------------- ")
    print (" MQTT authentication shortcut (default emtpy ='not required'):")
    print (" (authentication credentials are created/provided by MagPy's cred module)")
    newmqttcred = input()
    if newmqttcred:
        # check whether existing
        mqttcred = newmqttcred
        # if not create
        val = cred.lc(newmqttcred, "user")
        if not val:
            print("  ---- ")
            print("  Shortcut not yet existing - creating it: ")
            print("  Insert username of MQTT broker:")
            mqttuser = input()
            print("  Insert password of MQTT broker:")
            mqttpwd = getpass.getpass()
            #mqttpwd = input()
            cred.cc("transfer",newmqttcred,user=mqttuser,passwd=mqttpwd,address=mqttbroker,port=int(mqttport))
    print (" -> MQTT credentials: {}".format(mqttcred))


    print (" ------------------------------------------- ")
    print (" Notifications preference:")
    print ("  Select one of the following notification techniques: log, email, telegram.")
    print ("  Default is telegram.")
    newnot = input()
    notipath = os.path.join(confpath, "telegram.cfg")
    if newnot and newnot in ["log","email","telegram"]:
        # check whether existing
        noti = newnot
        print (" -> notification by: {}".format(noti))

    if noti == "email":
        print (" ------------------------------------------- ")
        print (" E-mail notifications:")
        print ("  Provide the addcred credential shortcut:")
        newmailcred = input()
        if newmailcred:
            # check whether existing
            mailcred = newmailcred
            val = cred.lc(newmailcred, "user")
            if not val:
                print (" ! Mail credentials do not yet exist - please create them to use email notifications")
            print (" -> E-mail credentials: {}".format(mailcred))
        notipath = os.path.join(confpath, "mail.cfg")
        print("  For email notification please update mail.cfg in your configuration directory.")
    elif noti == "telegram":
        print(" ------------------------------------------- ")
        print(" Telegram notifications:")
        print("  For telegram notification please update telegram.cfg in your configuration directory.")
        notipath = os.path.join(confpath, "telegram.cfg")
    else:
        pass

    print (" ------------------------------------------- ")
    print (" Two-way communication:")
    print ("  Do you want to establish a two-way communication using telegram?")
    print ("  Yes[y] or No[n] (default)")
    print ("  Please consider the manual for secure configuration and then update conf/telegrambot.cfg")
    print ("  You will also need to install telepot: pip install telepot")
    print ("  before using two communication.")
    newaddtelegrambot = input()
    if newaddtelegrambot in ["Yes", "YES", "Y", "y"]:
        addtelegrambot = True
        print(" -> creating a TELEGRAM BOT run time script (requires inputs in conf/telegrambot.cfg before running)")
        runscript = []
        runscript.append("#! /bin/bash")
        runscript.append("# TELEGRAM communication program")
        runscript.append("")
        runscript.append("{}".format(envact))
        runscript.append("")
        runscript.append('PYTHON={}'.format(sys.executable))
        runscript.append('BOT="telegrambot.py"')
        runscript.append('BOTPATH={}'.format(os.path.join(homedir, dir, "app")))
        runscript.append("")
        runscript.append('check_process()')
        runscript.append("{")
        runscript.append("    result=`/bin/ps aux | grep \"$BOT\" | grep -v grep | wc -l`")
        runscript.append("}")
        runscript.append('get_pid()')
        runscript.append("{")
        runscript.append("    pid=`/bin/ps -ef | grep \"$BOT\" | grep -v \"grep\" | awk '{print $2}'`")
        runscript.append("}")
        runscript.append("")
        runscript.append("check_process")
        runscript.append("# Run it")
        runscript.append("# ######")
        runscript.append("case \"$1\" in")
        runscript.append("  start)")
        runscript.append("    echo \"Starting $BOT ...\"")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \" $BOT is not running\"")
        runscript.append("        echo \" Starting $BOT\"")
        runscript.append("        echo \" --------------------\"")
        runscript.append("        sleep 2")
        runscript.append("        $PYTHON --version")
        runscript.append("        $PYTHON $BOTPATH/$BOT")
        runscript.append("    else")
        runscript.append("        echo \"$BOT is running already\"")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  stop)")
        runscript.append("    echo \"Stopping $BOT ...\"")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \" $BOT is not running\"")
        runscript.append("    else")
        runscript.append("        echo \" Stopping $BOT\"")
        runscript.append("        echo \" --------------------\"")
        runscript.append("        get_pid")
        runscript.append("        kill -9 $pid")
        runscript.append("        echo \" ... stopped\"")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  restart)")
        runscript.append("    echo \"Restarting $BOT ...\"")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"1\" ]; then")
        runscript.append("        echo \" Stopping $BOT\"")
        runscript.append("        get_pid")
        runscript.append("        kill -9 $pid")
        runscript.append("        echo \" ... stopped\"")
        runscript.append("    fi")
        runscript.append("    echo \" Starting $BOT\"")
        runscript.append("    echo \"--------------------\"")
        runscript.append("    sleep 2")
        runscript.append("    $PYTHON --version")
        runscript.append("    $PYTHON $BOTPATH/$BOT")
        runscript.append("    ;;")
        runscript.append("  status)")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \"$BOT is dead\"")
        runscript.append("    else")
        runscript.append("        echo \"$BOT is running\"")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  *)")
        runscript.append("    echo \"Usage: bash runbot.sh {start|stop|restart|status}\"")
        runscript.append("    ;;")
        runscript.append("esac")
        runscript.append("exit 0")
        with open(os.path.join(homedir, dir, "runbot.sh"), "wt") as fout:
            for line in runscript:
                fout.write(line+"\n")

    print (" ------------------------------------------- ")
    print (" Please select - you are initializing (A) a acquisition/MARTAS or (B) a collector/MARCOS")
    print (" ( select either A (default) or B )")
    initselect = input()
    if initselect in ["B","b"]:
        initjob = "MARCOS"
        print("  -> selected MARCOS")
    else:
        print("  -> selected MARTAS")

    monitorlog = os.path.join(os.path.join(logpath, "monitorstatus.log"))

    if initjob == "MARTAS":
        print (" ------------------------------------------- ")
        print (" Please insert a path for the sensor initialization files:")
        print (" (press return for accepting the default: {})".format(os.path.join(homedir,dir,"init")))
        newinitpath = input()
        if newinitpath:
            if not os.path.isdir(newinitpath):
                print("  ! the selected directory is not yet existing - aborting")
                sys.exit()
            if not os.access(newinitpath, os.W_OK):
                print(" ! you don't have write access to the specific directory - aborting")
                sys.exit()
            initpath = newinitpath
            shutil.copytree(os.path.join(homedir, dir, "init"), initpath)
        else:
            initpath = os.path.join(homedir, dir, "init")

        print (" ------------------------------------------- ")
        print (" Please insert a path for buffer files:")
        print (" (make sure it is existing and you have write permissions)".format(bufferpath))
        print (" (press return for accepting the default: {})".format(bufferpath))
        newbufferpath = input()
        if newbufferpath:
            bufferpath = newbufferpath
        if not os.path.isdir(bufferpath):
            print("  the selected directory is not yet existing - trying to create it...")
            try:
                Path(bufferpath).mkdir(parents=True, exist_ok=True)
                print("  done")
            except:
                print("  ! failed, check permissions - aborting")
                sys.exit()
        if not os.access(bufferpath, os.W_OK):
            print(" ! you don't have write access to the specific directory - aborting")
            sys.exit()

        print (" ------------------------------------------- ")
        print (" Please specify the MQTT payload format:")
        print (" (currently supported are 'martas' and 'intermagnet')")
        print (" ('intermagnet' is only available for mysql and imfile libraries)")
        print (" (default is 'martas')")
        newpayloadformat = input()
        if newpayloadformat == 'intermagnet':
            payloadformat = 'intermagnet'

        malogpath = os.path.join(logpath, "martas.log")

        print(" ------------------------------------------- ")
        print(" Creating the MARTAS run time script")
        runscript = []
        runscript.append("#! /bin/bash")
        runscript.append("# MARTAS acquisition program")
        runscript.append("")
        runscript.append("{}".format(envact))
        runscript.append("")
        runscript.append('PYTHON={}'.format(sys.executable))
        runscript.append('BOT="acquisition"')
        runscript.append('OPT="-m {}"'.format(os.path.join(confpath, "martas.cfg")))
        runscript.append("")
        runscript.append('check_process()')
        runscript.append("{")
        runscript.append("    result=`/bin/ps aux | grep \"$BOT $OPT\" | grep -v grep | wc -l`")
        runscript.append("}")
        runscript.append('get_pid()')
        runscript.append("{")
        runscript.append("    pid=`/bin/ps -ef | grep \"$BOT $OPT\" | grep -v \"grep\" | awk '{print $2}'`")
        runscript.append("}")
        runscript.append("")
        runscript.append("check_process")
        runscript.append("# Run it")
        runscript.append("# ######")
        runscript.append("case \"$1\" in")
        runscript.append("  start)")
        runscript.append("    echo \"Starting $BOT ...\" ")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \" $BOT is not running\" ")
        runscript.append("        echo \" Starting $BOT\"")
        runscript.append("        echo \" --------------------\"")
        runscript.append("        sleep 2")
        runscript.append("        $PYTHON --version")
        runscript.append("        $BOT $OPT")
        runscript.append("    else")
        runscript.append("        echo \"$BOT is running already\" ")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  stop)")
        runscript.append("    echo \"Stopping $BOT $OPT ...\" ")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \" $BOT is not running\" ")
        runscript.append("    else")
        runscript.append("        echo \" Stopping $BOT\"")
        runscript.append("        echo \" --------------------\"")
        runscript.append("        get_pid")
        runscript.append("        kill -9 $pid")
        runscript.append("        echo \" ... stopped\"")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  restart)")
        runscript.append("    echo \"Restarting $BOT $OPT ...\" ")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"1\" ]; then")
        runscript.append("        echo \" Stopping $BOT\" ")
        runscript.append("        get_pid")
        runscript.append("        kill -9 $pid")
        runscript.append("        echo \" ... stopped\"")
        runscript.append("    fi")
        runscript.append("    echo \" Starting $BOT\"")
        runscript.append("    echo \"--------------------\"")
        runscript.append("    sleep 2")
        runscript.append("    $PYTHON --version")
        runscript.append("    $BOT $OPT")
        runscript.append("    ;;")
        runscript.append("  status)")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \"$BOT $OPT is dead\" ")
        runscript.append("    else")
        runscript.append("        echo \"$BOT $OPT is running\" ")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  *)")
        runscript.append("    echo \"Usage: $BOT {start|stop|restart|status}\"")
        runscript.append("    ;;")
        runscript.append("esac")
        runscript.append("exit 0")
        with open(os.path.join(homedir, dir, "runmartas.sh"), "wt") as fout:
            for line in runscript:
                fout.write(line+"\n")

        print(" ------------------------------------------- ")
        print(" Creating the MARTAS viewer")
        viewscript = []
        viewscript.append("#! /bin/bash")
        viewscript.append("# MARTAS viewer")
        viewscript.append("")
        viewscript.append("pkill -f martas_view.py")
        viewscript.append('cd {}'.format(os.path.join(homedir,dir,"web")))
        viewscript.append('PYTHON={}'.format(sys.executable))
        viewscript.append('VIEW="{}"'.format(os.path.join(homedir,dir,"web","martas_view.py")))
        viewscript.append("$PYTHON $VIEW")
        viewscript.append("xdg-open http://127.0.0.1:8050")
        with open(os.path.join(homedir, dir, "martas_view"), "wt") as fout:
            for line in viewscript:
                fout.write(line+"\n")

        cronlist.append("# Running MARTAS ")
        cronlist.append("15 * * * *    /usr/bin/bash -i {} start > {} 2>&1".format(os.path.join(homedir, dir,"runmartas.sh"),os.path.join(homedir, dir, "log","runmartas.log")))
        cronlist.append("# Monitoring {} - hourly".format(jobname)) # jobname only for MARTAS

        with CronTab(user=True) as cron:
            comment = 'Running MARTAS'
            line = "/usr/bin/bash -i {} start > {} 2>&1".format(os.path.join(homedir, dir, "runmartas.sh"),
                                                          os.path.join(logpath, "runmartas.log"))
            if not list(cron.find_comment(comment)):
                job = cron.new(command=line, comment=comment)
                job.setall('15 * * * *')
            macomment2 = "Start MARTAS viewer"
            maline2 = "/usr/bin/bash -i {} > {} 2>&1".format(os.path.join(homedir, dir, "martas_view"),
                                                          os.path.join(logpath, "martas_view.log"))
            if not list(cron.find_comment(macomment2)):
                job2 = cron.new(command=maline2, comment=macomment2)
                job2.setall('16 0 * * *')
                job2.enable(False)

        #print('cron.write() was just executed')

    elif initjob == "MARCOS":
        print (" You can have multiple collector jobs on one machine.")
        print (" Make sure they have different names.")
        print (" ------------------------------------------- ")
        print (" Please insert a name for the collector:")
        print (" (press return for accepting default: {})".format(jobname))
        print (" (the given name will be extended 'collect-NAME')")
        print (" (ideally you provide the name of the MARTAS from which you are collecting data)")
        newjobname = input()
        if newjobname:
            jobname = ''.join(filter(str.isalnum, newjobname))
        print (" -> MARCOS job name: {}".format(jobname))
        marcosjob = "collect-{}".format(jobname)

        print (" ------------------------------------------- ")
        print (" Please specify an output destination:")
        print (" (choose from stdout, db, file, websocket, diff)")
        print (" (multiple selections i.e. file,db are possible)")
        newdestination = input()
        if newdestination:
            valid = ["stdout", "db", "file", "websocket", "diff"]
            if any(s in newdestination for s in valid):
                pass
            else:
                print("  ! invalid destination - aborting")
                sys.exit()
            destination = newdestination

        if destination.find("file") >= 0:
            print (" ------------------------------------------- ")
            print (" Please specify a path to store files:")
            filepath = input()
            if not os.path.isdir(filepath):
                print ("  ! the selected directory is not yet existing - please create and re-run martas-init - aborting now")
                sys.exit()
            if not os.access(filepath, os.W_OK):
                print (" ! you don't have write access - aborting")
                sys.exit()

        if destination.find("db") >= 0:
            print (" ------------------------------------------- ")
            print (" Please specify credentials (see addcred) for database:")
            newdatabasecredentials = input()
            if newdatabasecredentials:
                val = cred.lc(newdatabasecredentials, "user")
                if not val:
                    print (" ! Database credentials do not exist")
                databasecredentials = newdatabasecredentials

        print (" ------------------------------------------- ")
        print (" MARCOS will archive database contents in CDF files by default.")
        print(" Please provide a path for the data archive (default: {}):".format(archivepath))
        newarchivepath = input()
        if newarchivepath:
            archivepath = newarchivepath
        if not os.path.isdir(archivepath):
            print("  the selected directory is not yet existing - trying to create it...")
            try:
                Path(archivepath).mkdir(parents=True, exist_ok=True)
                print("  done")
            except:
                print("  ! failed, check permissions and re-run martas-init - aborting")
                sys.exit()
        if not os.access(archivepath, os.W_OK):
            print (" ! you don't have write access - select an appropriate path and re-run martas-init - aborting")
            sys.exit()

        print(" ------------------------------------------- ")
        print(" Threshold tester can be used on archive or db.")
        defaultsource = "db"
        if not destination.find("db") >= 0:
            defaultsource = "file"
            print(" Please enter 'db' or 'file' (default):")
        else:
            print(" Please enter 'db' (default) or 'file':")
        newthresholdsource = input()
        if newthresholdsource in ['db','file']:
            thresholdsource = newthresholdsource
        else:
            thresholdsource = defaultsource

        malogpath = os.path.join(logpath, "{}.log".format(marcosjob))
        archivelog = os.path.join(logpath, "archivestatus.log")

        print(" ------------------------------------------- ")
        print(" Creating the MARCOS run time script for {}".format(jobname))
        runscript = []
        runscript.append("#! /bin/bash")
        runscript.append("# MARCOS acquisition program")
        runscript.append("")
        runscript.append("{}".format(envact))
        runscript.append("")
        runscript.append('PYTHON={}'.format(sys.executable))
        runscript.append('BOT="collector"')
        runscript.append('OPT="-m {}"'.format(os.path.join(confpath, "{}.cfg".format(marcosjob))))
        runscript.append("")
        runscript.append('check_process()')
        runscript.append("{")
        runscript.append("    result=`ps aux | grep \"$BOT $OPT\" | grep -v grep | wc -l`")
        runscript.append("}")
        runscript.append('get_pid()')
        runscript.append("{")
        runscript.append("    pid=`/bin/ps -ef | grep \"$BOT $OPT\" | grep -v \"grep\" | awk '{print $2}'`")
        runscript.append("}")
        runscript.append("")
        runscript.append("check_process")
        runscript.append("# Run it")
        runscript.append("# ######")
        runscript.append("case \"$1\" in")
        runscript.append("  start)")
        runscript.append("    echo \"Starting $BOT $OPT ...\" ")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \" $BOT is not running\" ")
        runscript.append("        echo \" Starting $BOT\"")
        runscript.append("        echo \" --------------------\"")
        runscript.append("        sleep 2")
        runscript.append("        $PYTHON --version")
        runscript.append("        $BOT $OPT")
        runscript.append("    else")
        runscript.append("        echo \"$BOT is running already\" ")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  stop)")
        runscript.append("    echo \"Stopping $BOT $OPT ...\" ")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \" $BOT is not running\" ")
        runscript.append("    else")
        runscript.append("        echo \" Stopping $BOT\"")
        runscript.append("        echo \" --------------------\"")
        runscript.append("        get_pid")
        runscript.append("        kill -9 $pid")
        runscript.append("        echo \" ... stopped\"")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  restart)")
        runscript.append("    echo \"Restarting $BOT $OPT ...\" ")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"1\" ]; then")
        runscript.append("        echo \" Stopping $BOT\" ")
        runscript.append("        get_pid")
        runscript.append("        kill -9 $pid")
        runscript.append("        echo \" ... stopped\"")
        runscript.append("    fi")
        runscript.append("    echo \" Starting $BOT\"")
        runscript.append("    echo \"--------------------\"")
        runscript.append("    sleep 2")
        runscript.append("    $PYTHON --version")
        runscript.append("    $BOT $OPT")
        runscript.append("    ;;")
        runscript.append("  update)")
        runscript.append("    echo \"Starting $BOT $OPT with meta information update...\" ")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"1\" ]; then")
        runscript.append("        echo \" Stopping $BOT\" ")
        runscript.append("        get_pid")
        runscript.append("        kill -9 $pid")
        runscript.append("        echo \" ... stopped\"")
        runscript.append("    fi")
        runscript.append("    echo \" Starting $BOT\"")
        runscript.append("    echo \"--------------------\"")
        runscript.append("    sleep 2")
        runscript.append("    $PYTHON --version")
        runscript.append("    $BOT $OPT -v")
        runscript.append("    ;;")
        runscript.append("  status)")
        runscript.append("    check_process")
        runscript.append("    if [ \"$result\" = \"0\" ]; then")
        runscript.append("        echo \"$BOT $OPT is dead\" ")
        runscript.append("    else")
        runscript.append("        echo \"$BOT $OPT is running\" ")
        runscript.append("    fi")
        runscript.append("    ;;")
        runscript.append("  *)")
        runscript.append("    echo \"Usage: $BOT {start|stop|restart|status|update}\"")
        runscript.append("    ;;")
        runscript.append("esac")
        runscript.append("exit 0")
        with open(os.path.join(homedir, dir, "{}.sh".format(marcosjob)), "wt") as fout:
            for line in runscript:
                fout.write(line+"\n")

        print(" ------------------------------------------- ")
        print(" Creating the MARCOS viewer")
        viewscript = []
        viewscript.append("#! /bin/bash")
        viewscript.append("# MARCOS viewer")
        viewscript.append("")
        viewscript.append("pkill -f marcos_view.py")
        viewscript.append('cd {}'.format(os.path.join(homedir,dir,"web")))
        viewscript.append('PYTHON={}'.format(sys.executable))
        viewscript.append('VIEW="{}"'.format(os.path.join(homedir,dir,"web","marcos_view.py")))
        viewscript.append("$PYTHON $VIEW")
        viewscript.append("xdg-open http://127.0.0.1:8050")
        with open(os.path.join(homedir, dir, "marcos_view"), "wt") as fout:
            for line in viewscript:
                fout.write(line+"\n")


        with CronTab(user=True) as cron:
            comment1 = "Running MARCOS process {}".format(jobname)
            line1 = "/usr/bin/bash -i {} start > {} 2>&1".format(os.path.join(homedir, dir, marcosjob+".sh"),os.path.join(logpath, marcosjob+".log"))
            if not list(cron.find_comment(comment1)):
                job1 = cron.new(command=line1, comment=comment1)
                job1.setall('17 * * * *')
            comment2 = "Archiving"
            line2 = "{} {} -c {} > {} 2>&1".format(sys.executable, os.path.join(homedir, dir,"app","archive.py"),os.path.join(confpath,"archive.cfg"), os.path.join(logpath,"archive.log"))
            if not list(cron.find_comment(comment2)):
                job2 = cron.new(command=line2, comment=comment2)
                job2.setall('20 0 * * *')
            comment2b = "Truncating old and none-DATAINFO tables"
            line2b = "{} {} -c {} > {} 2>&1".format(sys.executable, os.path.join(homedir, dir,"app","db_truncate.py"),os.path.join(confpath,"archive.cfg"), os.path.join(logpath,"db_truncate.log"))
            if not list(cron.find_comment(comment2b)):
                job2b = cron.new(command=line2b, comment=comment2b)
                job2b.setall('20 2 * * *')
            comment2a = "Filtering"
            line2a = "{} {} -c {} > {} 2>&1".format(sys.executable, os.path.join(homedir, dir,"app","filter.py"),os.path.join(confpath,"filter.cfg"), os.path.join(logpath,"filter.log"))
            if not list(cron.find_comment(comment2a)):
                job2a = cron.new(command=line2a, comment=comment2a)
                job2a.minute.every(2)
                job2a.enable(False)
            if destination.find("db") >= 0:
                comment3 = "Optimizing database"
                line3 = "{} {} -c {} -s sqlmaster > {} 2>&1".format(sys.executable, os.path.join(homedir, dir,"app","optimizetables.py"), os.path.join(confpath,"archive.cfg"), os.path.join(logpath,"optimizetables.log"))
                if not list(cron.find_comment(comment3)):
                    job3 = cron.new(command=line3, comment=comment3)
                    job3.setall('2 0 * * 2')
            mvcomment2 = "Start MARCOS viewer"
            mvline2 = "/usr/bin/bash -i {} > {} 2>&1".format(os.path.join(homedir, dir, "marcos_view"),
                                                          os.path.join(logpath, "marcos_view.log"))
            if not list(cron.find_comment(mvcomment2)):
                jobmv2 = cron.new(command=mvline2, comment=mvcomment2)
                jobmv2.setall('16 0 * * *')
                jobmv2.enable(False)
            mvcomment3 = "Optional: Regular download of buffer from MARCOS {}".format(jobname)
            mvline3 = "{} {} -c {} > {} 2>&1".format(sys.executable, os.path.join(homedir, dir,"app","file_download.py"),os.path.join(confpath,"download-{}.cfg".format(jobname)), os.path.join(logpath,"download-{}.log".format(jobname)))
            if not list(cron.find_comment(mvcomment3)):
                jobmv3 = cron.new(command=mvline3, comment=mvcomment3)
                jobmv3.setall('16 0 * * *')
                jobmv3.enable(False)

    with CronTab(user=True) as cron:
        if initjob == "MARCOS":
            comment4 = "Monitoring MARCOS - hourly"
        else:
            comment4 = "Monitoring MARTAS {} - hourly".format(jobname)
        line4 = "{} {} -c {} > {} 2>&1".format(sys.executable, os.path.join(homedir, dir,"app","monitor.py"),os.path.join(confpath,"monitor.cfg"), os.path.join(logpath,"monitor.log"))
        if not list(cron.find_comment(comment4)):
            job4 = cron.new(command=line4, comment=comment4)
            job4.setall('30 * * * *')
        comment5 = "Running cleanup"
        line5 = "/usr/bin/bash -i {} > {} 2>&1".format(os.path.join(homedir, dir, "scripts", "cleanup.sh"),
                                                                              os.path.join(logpath,
                                                                                           "cleanup.log"))
        if not list(cron.find_comment(comment5)):
            job5 = cron.new(command=line5, comment=comment5)
            job5.setall('9 0 * * *')

        if backuppath:
            comment6 = "Backup of MARTAS"
            line6 = "{} {} -b {} -d {} > {} 2>&1".format(sys.executable, os.path.join(homedir, dir,"app","backup.py"), os.path.join(homedir, dir), backuppath, os.path.join(logpath,"backup.log"))
            if not list(cron.find_comment(comment6)):
                job6 = cron.new(command=line6, comment=comment6)
                job6.setall('7 0 * * 1')
        comment7 = "Threshold testing - please configure before enabling"
        line7 = "{} {} -m {}".format(sys.executable, os.path.join(homedir, dir,"app","threshold.py"), os.path.join(confpath,"threshold.cfg"))
        if not list(cron.find_comment(comment7)):
            job7 = cron.new(command=line7, comment=comment7)
            job7.minute.every(10)
            job7.enable(False)
        comment8 = "Log rotation for {}".format(jobname)
        line8 = "/usr/sbin/logrotate -s {} {} > /dev/null 2>&1".format(os.path.join(homedir, dir, "logrotate", "logstate"), os.path.join(homedir, dir, "logrotate", "{}.logrotate".format(jobname)))
        if not list(cron.find_comment(comment8)):
            job8 = cron.new(command=line8, comment=comment8)
            job8.setall('3 1 * * *')

        if addtelegrambot:
            comment10 = "Telegram two-way communication bot"
            comment10c = "Telegram two-way communication bot - restart"
            line10 = "/usr/bin/bash -i {} start 2>&1".format(os.path.join(homedir, dir, "runbot.sh"))
            line10c = "/usr/bin/bash -i {} restart 2>&1".format(os.path.join(homedir, dir, "runbot.sh"))
            if not list(cron.find_comment(comment10)):
                job10a = cron.new(command=line10, comment=comment10)
                job10a.setall('0 8 * * *')
                job10b = cron.new(command=line10, comment=comment10)
                job10b.setall('0 16 * * *')
                job10c = cron.new(command=line10c, comment=comment10c)
                job10c.setall('15 1 * * *')


    replacedict = { "/logpath" : malogpath,
                    "/sensorpath" : os.path.join(confpath, "sensors.cfg"),
                    "/initdir" : initpath,
                    "/mainpath" : os.path.join(homedir,dir),
                    "/homedirectory" : homedir,
                    "/srv/mqtt" : bufferpath,
                    "/obsdaqpath" : os.path.join(confpath, "obsdaq.cfg"),
                    "myhome" : stationname,
                    "outputdestination" : destination,
                    "filepath  :  /tmp" : "filepath  :  {}".format(filepath),
                    "/archivepath" : archivepath,
                    "archivelog" : archivelog,
                    "monitorlog" : monitorlog,
                    "thresholdlog" : os.path.join(logpath,"thresholdstatus.log"),
                    "collectlog" : os.path.join(logpath,"download-source.log"),
                    "filterlog" : os.path.join(logpath,"filterstatus.log"),
                    "basevaluelog" : os.path.join(logpath,"basevaluestatus.log"),
                    "/telegrambotlogpath" : os.path.join(logpath,"telegrambot.log"),
                    "thresholdsource" : thresholdsource,
                    "mynotificationtype" : noti,
                    "notificationcfg" : notipath,
                    "mymailcred" : mailcred,
                    "mydb" : databasecredentials,
                    "./web" : "{}".format(os.path.join(homedir, dir,"web")),
                    "brokeraddress" : mqttbroker,
                    "1883"  :  mqttport,
                    "mqttqos  :  0": "mqttqos  :  {}".format(mqttqos),
                    "payloadformat  :  martas": "payloadformat  :  {}".format(payloadformat),
                    "/home/username/.magpycred" : os.path.join(homedir, ".magpycred"),
                    }
    if mqttcred:
        replacedict["#mqttcred  :  shortcut"] = "mqttcred  :  {}".format(mqttcred)
    if mqttcert:
        replacedict["#mqttcert  :  /path/to/ca.crt"] = "mqttcert  :  {}".format(mqttcert)
    if pskcredentials:
        replacedict["#mqttpsk  :  pskcredentials"] = "mqttpsk  :  {}".format(pskcredentials)
    if backuppath:
        replacedict["/backuppath"] = backuppath

    files_to_change = {}
    if initjob == "MARTAS":
        replacedict["/mybasedir"] = bufferpath
        replacedict["space,martas,marcos,logfile"] = "space,martas,logfile"
        files_to_change["martasconf"] = {"source" : os.path.join(confpath, "martas.bak") ,
                                        "dest" : os.path.join(confpath, "martas.cfg") }
        replacedict["homedirectory"] = 'martas'
        files_to_change["webconf"] = {"source" : os.path.join(confpath, "web.bak") ,
                                        "dest" : os.path.join(confpath, "web.cfg") }

    if initjob == "MARCOS":
        replacedict["/mybasedir"] = archivepath
        replacedict["space,martas,marcos,logfile"] = "space,marcos"
        files_to_change["marcosconf"] = {"source" : os.path.join(confpath, "marcos.bak") ,
                                        "dest" : os.path.join(confpath, "{}.cfg".format(marcosjob)) }
        files_to_change["archiveconf"] = {"source": os.path.join(confpath, "archive.bak"),
                                     "dest": os.path.join(confpath, "archive.cfg")}
        files_to_change["basevalueconf"] = {"source": os.path.join(confpath, "basevalue.bak"),
                                     "dest": os.path.join(confpath, "basevalue.cfg")}
        files_to_change["filterconf"] = {"source": os.path.join(confpath, "filter.bak"),
                                         "dest": os.path.join(confpath, "filter.cfg")}
        files_to_change["downloadconf"] = {"source": os.path.join(confpath, "download-source.bak"),
                                           "dest": os.path.join(confpath, "download-{}.cfg".format(jobname))}
        replacedict["homedirectory"] = 'marcos'
        files_to_change["webconf"] = {"source" : os.path.join(confpath, "web.bak") ,
                                        "dest" : os.path.join(confpath, "web.cfg") }
        # file for which replacements will happen and new names
    files_to_change["skeletonlogrotate"] = {"source": os.path.join(homedir, dir, "logrotate", "skeleton.logrotate"),
                              "dest": os.path.join(homedir, dir, "logrotate", "{}.logrotate".format(jobname))}
    files_to_change["fileuploadconf"] = {"source": os.path.join(confpath, "fileuploadjobs.bak"),
                        "dest": os.path.join(confpath, "fileuploadjobs.json")}
    files_to_change["monitorconf"] = {"source": os.path.join(confpath, "monitor.bak"),
                        "dest": os.path.join(confpath, "monitor.cfg")}
    files_to_change["thresholdconf"] = {"source": os.path.join(confpath, "threshold.bak"),
                        "dest": os.path.join(confpath, "threshold.cfg")}
    files_to_change["mailconf"] = {"source": os.path.join(confpath, "mail.bak"),
                        "dest": os.path.join(confpath, "mail.cfg")}
    files_to_change["gammaconf"] = {"source": os.path.join(confpath, "gamma.bak"),
                        "dest": os.path.join(confpath, "gamma.cfg")}
    files_to_change["cleanup"] = {"source": os.path.join(homedir, dir, "scripts", "cleanup.bak"),
                        "dest": os.path.join(homedir, dir, "scripts", "cleanup.sh")}
    files_to_change["telegrambot"] = {"source": os.path.join(confpath, "telegrambot.bak"),
                        "dest": os.path.join(confpath, "telegrambot.cfg")}


    for f in files_to_change:
        d = files_to_change.get(f)
        with open(d.get("source"), "rt") as fin:
            with open(d.get("dest"), "wt") as fout:
                for line in fin:
                    for el in replacedict:
                        if debug:
                            print ("Replacing", el, replacedict.get(el))
                        line = line.replace(el, replacedict.get(el))
                    fout.write(line)

    print("Crontab looks like:")
    print("\n".join(cronlist))
    print("Setup finished. SUCCESS")  # used for monitoring of logfile
    # end of init
    print("")  # used for monitoring of logfile
    print("Things to do:")
    if initjob == "MARTAS":
        print("- update sensors.cfg and add your connected sensors in {}".format(os.path.join(homedir,dir,"conf")))
        print("- eventually update your sensors initialization script if required")
    print("- check configuration files in {}".format(os.path.join(homedir,dir,"conf")))
    if initjob == "MARCOS":
        print("- update filter.cfg and activate filter job in crontab in case of > 1Hz data")
        print("- read the manual (again) for monitoring and threshold - activate/configure")
        print("- if you want regular downloads of buffer files in addition to MQTT then update the download job")
        print("- your collector job will be started tonight. Please start manually with upload option:")
        print("  bash collect-{}.sh update".format(jobname))
        print("  keep running for a few minutes. This will update the meta information in your database.")
    print("- you might want to add job restarts after rebooting into crontab:")
    print("  @reboot sleep 60 && bash -i /jobpath/runmartas.sh start > /logpath/runmartas.log 2>&1")
    print("- you also might want to add paths to crontab PATH=/bin:/usr/bin:/usr/local/bin")
    print("  which at least is necessary if you are using 2-way comm")

if __name__ == "__main__":
   main(sys.argv[1:])



#!/usr/bin/env python
"""
Regularly archive data from a database
use crontab or scheduler to apply archive methods
"""

from magpy.stream import *
from magpy.core import database
from magpy.core import flagging
from magpy.core import methods as magpymeth

from magpy.opt import cred as mpcred


# Relative import of core methods as long as martas is not configured as package
from martas.core.methods import martaslog as ml
from martas.core import methods as mm
from martas.version import __version__


import getopt
from datetime import datetime, timezone
import pwd
import socket


"""
#configuration:

dbcredentials     :      cobsdb

archivepath            :      /srv/archive

# By default all sensors in path will be used with default criteria
defaultdepth    :      2

archiveformat   :      PYCDF

writearchive    :      True
subdirectory    :      Y

applyflags      :      False

cleandb         :      True

# cleanratio = samplingrateratio for deleting old db entries - default is 12')
#                   deleting data older than samplingrate(sec)*12 days.')
#                    cleanratio=12 : 1sec data older than 12 days is deleted in DB')
#                                  :           1min data older than 720 days is deleted in DB')
#                    cleanratio=1  : 1sec data older than 1 day is deleted in DB')
#                                  :           1min data older than 60 days is deleted in DB
cleanratio      :      12


# Modify criteria for specific sensors
sensordict      :    Sensor1:depth,format,writeDB,writeArchive,applyFlags,cleanratio;


# Sensors present in path to be skipped (Begging of Sensorname is enough
blacklist       :    BLV,QUAKES,Sensor2,Sensor3,



DESCRIPTION:
    archive.py gets data from a databank
    to any accessible repository (e.g. disk).
    Old database entries exceding a defined age
    are deleted. Optionally archive files can be stored in a user defined format.
    The databank size is automatically restricted
    in dependency of the sampling rate of the input data. A cleanratio of 12 
    will only keep the last
    12 days of second data, the last 720 days of minute data and
    approximately 118 years of hourly data are kept.
    Settings are given in a configuration file.
    IMPORTANT: take care about depth - needs to be large enough to find data

    For reading raw data and creating archive file please use something else 

APPLICATION:
    # Auomatic
    python3 archive.py -c config.cfg

    # Manual for specific sensors and time range
    python3 archive.py -c /config.cfg -b 2020-11-22 -s Sensor1,Sensor2 -d 30

"""


def create_datelist(startdate='', depth=2, debug=False):

    if debug:
         print ("   Creating datelist...")
    # Getting dates
    datelist = []
    newdatetuple = []
    if startdate == '':
        current = datetime.now(timezone.utc).replace(tzinfo=None)
    else:
        current = magpymeth.testtime(startdate)

    newcurrent = current
    for elem in range(depth):
        datelist.append(newcurrent.strftime("%Y-%m-%d"))
        td = current-timedelta(days=elem+1)
        datetuple = (td.strftime("%Y-%m-%d"), newcurrent.strftime("%Y-%m-%d"))
        newcurrent = current-timedelta(days=elem+1)
        newdatetuple.append(datetuple)
    if debug:
         print ("   -> ", newdatetuple)

    return newdatetuple


def create_data_selectionlist(blacklist=None, debug=False):
    if not blacklist:
        blacklist = []
    addstr = ""
    if debug:
         print ("   Creating sql query...")
    # get a list with all datainfoids covering the selected time range
    sql = 'SELECT DataID,DataMinTime,DataMaxTime FROM DATAINFO'

    if len(blacklist) > 0:
        sql = "{} WHERE".format(sql)
        for el in blacklist:
            addstr += " AND DataID NOT LIKE '{}%'".format(el)
        addstr = addstr.replace(" AND","",1)
    sql = "{}{}".format(sql,addstr)

    if debug:
         print ("   -> query looks like: {}".format(sql))
    return sql


def get_data_dictionary(db,sql,debug=False):

    if debug:
         print ("   Obtaining DataID dict with times...")

    resultdict = {}
    # looks like {'data_1_0001_0001' : {'mintime':xxx, 'maxtime' : xxx }, ...}

    cursor = db.db.cursor()
    try:
        cursor.execute(sql)
    except:
        print ("   Error when sending sql query")
    result =  cursor.fetchall()
    for el in result:
        # check whether a data table with this name is existing
        verifytable = "SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = '{}'".format(el[0])
        cursor.execute(verifytable)
        test =  cursor.fetchall()
        if test:
            resultdict[el[0]] = {'mintime': el[1], 'maxtime': el[2]}
    cursor.close()
    if debug:
        print ("   -> Obtained the following DataIDs:", resultdict)

    return resultdict

def get_parameter(plist, debug=False):
    if debug:
        print ("Found the following parameters: {}".format(plist))
    try:
        depth = int(plist[0])
    except:
        print ("  Depth needs to be an integer")
        sys.exit()
    fo = plist[1]
    wa = mm.get_bool(plist[2])
    af = mm.get_bool(plist[3])
    cdb = mm.get_bool(plist[4])
    try:
        ratio = int(plist[5])
    except:
        print ("  Ratio needs to be an integer")
        sys.exit()
    return depth,fo,wa,af,cdb,ratio


def validtimerange(timetuple, mintime, maxtime, debug=False):
    #timetuple to 1d list unique list
    mintuptime = magpymeth.testtime(min(min(timetuple)))
    maxtuptime = magpymeth.testtime(max(max(timetuple)))
    mintime = magpymeth.testtime(mintime)
    maxtime = magpymeth.testtime(maxtime)
    if mintuptime < maxtime and maxtuptime > mintime:
        if debug:
            print ("  Found valid time range")
        return True
    if debug:
        print ("  Found invalid time range")
    return False

def main(argv):
    version = __version__
    conf = ''
    path = ''
    obsdepth = 0
    obssenslist = []
    startdate = ''
    statusmsg = {}
    proxies = {}
    hostname = socket.gethostname().upper()
    debug=False
    try:
        opts, args = getopt.getopt(argv,"hc:b:d:s:gi:a:D",["config=","begin=","depth=","sensors=","debug=",])
    except getopt.GetoptError:
        print ('archive.py -c <config> -b <begin> -d <depth> -s <sensors>')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print ('-------------------------------------')
            print ('Description:')
            print ('-- archive.py gets data from a databank and archives  --')
            print ('-- to any accessible repository (e.g. disk)           --')
            print ('Old database entries exceding a defined age')
            print ('are deleted. Optionally archive files can be stored in a user defined format.')
            print ('The databank size is automatically restricted ')
            print ('in dependency of the sampling rate of the input data. Only the last ')
            print ('12 days of second data, the last 720 days of minute data and ')
            print ('approximately 118 years of hourly data are kept. To modify these default')
            print ('settings please contact the developers (or learn python and')
            print ('edit the code - its simple and the MagPy cookbook will help you).')
            print ('IMPORTANT: data bank entries are solely identified from DATAINFO table.')
            print ('           make sure that your data tables are contained there')
            print ('IMPORTANT: take care about depth - needs to be large enough to find data')
            print ('-------------------------------------')
            print ('Usage:')
            print ('archive.py -c <config> -p <archivepath> -b <begin> -d <depth> -s <sensors>')
            print ('-------------------------------------')
            print ('Options:')
            print ('-c (required) : provide a path to a configuartion file')
            print ('-p            : archivepath like "/home/max/myarchive"')
            print ('              : please note: an asterix requires quotes')
            print ('-d            : depth')
            print ('-b            : begin: end = begin - depth(days)')
            print ('-s            : list sensor to deal with')
            print ('-------------------------------------')
            print ('Example:')
            print ('every day cron job: python archive.py -c cobsdb -p /srv/archive')
            print ('creating archive of old db entries: python archive.py -c cobsdb -p /media/Samsung/Observatory/data/ -d 30 -b "2012-06-01" -g -i 100 -a 3')
            sys.exit()
        elif opt in ("-c", "--config"):
            conf = os.path.abspath(arg)
        elif opt in ("-b", "--begin"):
            startdate = arg
        elif opt in ("-d", "--depth"):
            try:
                obsdepth = int(arg)
                if not obsdepth >= 2:
                    print ("depth needs to be positve and larger or equal than 2")
                    sys.exit()
            except:
                print ("depth needs to be an integer")
                sys.exit()
        elif opt in ("-s", "--sensors"):
            obssenslist = arg.split(",")
        elif opt == "-v":
            print ("archive.py version: {}".format(version))
        elif opt in ("-D", "--debug"):
            debug = True

    print ("Running archive.py")
    print ("-------------------------------")

    if conf == '':
        print ('Specify a path to a configuration file using the  -c option:')
        print ('-- check archive.py -h for more options and requirements')
        sys.exit()
    else:
        if os.path.isfile(conf):
            print ("  Read file with GetConf")
            config = mm.get_conf(conf)
            print ("   -> configuration data extracted")
        else:
            print ('Specify a valid path to a configuration file using the  -c option:')
            print ('-- check archive.py -h for more options and requirements')
            sys.exit()

    ## Logger configuration data
    logpath = config.get('logpath')
    receiver = config.get('notification')
    receiverconf = config.get('notificationconf')
    subdirectory = config.get('subdirectory',None)

    db = mm.connect_db(config.get('dbcredentials'))

    sql = create_data_selectionlist(blacklist=config.get('blacklist',[]), debug=debug)

    datainfoiddict = get_data_dictionary(db,sql,debug=False)

    for data in datainfoiddict:
        sr = 1
        datainfoid = ''
        print (" ---------------------------- ")
        print (" Checking data set {}".format(data))
        name = "{}-archiving-{}".format(hostname,data.replace("_","-"))
        msg = "checking"
        if debug:
            print ("  Times: {}".format(datainfoiddict.get(data)))
        # TODO create a warning if mintime is much younger as it should be after cleaning

        if obssenslist and not data in obssenslist:
            print ("  Not in observers specified dataid list - this DataID will be skipped")
            continue

        if not db: # check whether db is still connected
            print ("    Lost DB - reconnecting ...")
            db = mm.connect_db(config.get('dbcredentials'), exitonfailure=False, report=False)

        para = [config.get('defaultdepth'), config.get('archiveformat'),config.get('writearchive'),config.get('applyflags'), config.get('cleandb'),config.get('cleanratio')]
        # Get default parameter from config
        depth,fo,wa,af,cdb,ratio = get_parameter(para)
        writemode = config.get('writemode','replace')

        # Modify parameters if DataID specifications are give
        for sensd in config.get('sensordict',{}):
            if data.find(sensd) >= 0:
                print ("  Found data specific parameters for sensorgroup {}:".format(sensd)) 
                para = config.get('sensordict').get(sensd)
                depth,fo,wa,af,cdb,ratio = get_parameter(para)
                print ("   -> {}".format(para))

        # Manual specifications
        if obsdepth:
            print ("  Overriding configuration file data with manual specifications - new depth = {}".format(obsdepth)) 
            depth = obsdepth

        # Create datelist (needs to be sorted)
        dateslist = create_datelist(startdate=startdate, depth=depth, debug=debug)

        # check time range
        try:
            # This method might fail if datainfodict does not contain dates - in this case just proceed with normal analysis
            gettrstate = validtimerange(dateslist, datainfoiddict.get(data).get('mintime'), datainfoiddict.get(data).get('maxtime'))
        except:
            print ("   -> Could not extract time ranges from datainfo dictionary")
            gettrstate = True

        if not gettrstate:
            print ("  Apparently no data is existing for the seleceted days - skipping")
            continue

        # run the following in a daily manner? to save memory... check
        for tup in dateslist:
            if debug:
                print ("  Running for range", tup)
            stream = DataStream()
            #tup = (day,nextday)
            if debug:
                print ("  Reading data from DB ...")
            stream = db.read(data,starttime=tup[0],endtime=tup[1])
            if debug:
                print ("    -> Done ({} data points)".format(stream.length()[0]))

            # Data found
            if stream.length()[0] > 0:
                dataidnum = stream.header.get('DataID')
                if not max(stream.ndarray[0]).replace(tzinfo=None) < datetime.now(timezone.utc).replace(tzinfo=None):
                    print ("  Found in-appropriate date in stream - maxdate = {} - cutting off".format(max(stream.ndarray[0])))
                    stream = stream.trim(endtime=datetime.now(timezone.utc).replace(tzinfo=None))
                print ("  Archiving {} data from {} to {}".format(data,tup[0],tup[1]))
                sr = stream.samplingrate()
                print ("   with sampling period {} sec".format(sr))
                if isnan(sr):
                    print ("Please take care - could not extract sampling rate - will assume 60 seconds")
                    sr = 60

                path = config.get('archivepath')
                archivepath = None

                if path:
                    #construct archive path
                    try:
                        sensorid = stream.header['SensorID']
                        stationid = stream.header['StationID']
                        datainfoid = stream.header['DataID']
                        archivepath = os.path.join(path,stationid,sensorid,stream.header['DataID'])
                    except:
                        print ("  Obviously a problem with insufficient header information")
                        print ("  - check StationID, SensorID and DataID in DB")
                        archivepath = None

                if af and sr > 0.9:
                    print ("You selected to apply flags and save them along with the cdf archive.")
                    flaglist = db.flags_from_db(sensorid=stream.header['SensorID'],starttime=tup[0],endtime=tup[1])
                    if len(flaglist) > 0:
                        print ("  Found {} flags in database for the selected time range - adding them to the archive file".format(len(flaglist)))
                        stream = flaglist.apply_flags(stream, mode='insert')

                if not debug and wa and archivepath:
                    print ("Writing data to:", archivepath)
                    stream.write(archivepath,filenamebegins=datainfoid+'_',format_type=fo,mode=writemode,subdirectory=subdirectory)
                    print ("... success")
                else:
                    print ("   Debug: skip writing")
                    print ("    -> without debug a file with {} inputs would be written to {}".format(stream.length()[0],archivepath))
            else:
                print ("No data between {} and {}".format(tup[0],tup[1]))
            msg = "successfully finished"

        if not debug and cdb and not datainfoid == '':
                    print ("Now deleting old entries in database older than {} days".format(sr*ratio))
                    # TODO get coverage before
                    db.delete(datainfoid,samplingrateratio=ratio)
                    # TODO get coverage after
        else:
                    print ("   Debug: skip deleting DB")
                    print ("    -> without debug all entries older than {} days would be deleted".format(sr*ratio))

        statusmsg[name] = msg

    if debug or obssenslist:   #No update of statusmessages if only a selected sensor list is analyzed
        print (statusmsg)
    else:
        if config.get('https'):
            proxies['https'] = config.get('https')
        if config.get('http'):
            proxies['http'] = config.get('http')
        martaslog = ml(logfile=logpath,receiver=receiver,proxies=proxies)
        martaslog.telegram['config'] = receiverconf
        martaslog.msg(statusmsg)

    print ("----------------------------------------------------------------")
    print ("archiving app finished")
    print ("----------------------------------------------------------------")
    print ("SUCCESS")

if __name__ == "__main__":
   main(sys.argv[1:])






#!/usr/bin/env python

from __future__ import print_function
from __future__ import absolute_import

# ###################################################################
# Import packages
# ###################################################################

import serial
import binascii
import csv
import sys, getopt
import time
import os
import re     # for interpretation of lines
import struct # for binary representation
import socket # for hostname identification
import string # for ascii selection
from datetime import datetime, timedelta
from twisted.python import log

SENSORELEMENTS =  ['sensorid','port','baudrate','bytesize','stopbits', 'parity','mode','init','rate','stack','protocol','name','serialnumber','revision','path','pierid','ptime','sensorgroup','sensordesc']


def connect(parameter):
    # Connecting
    print ("Welcome to serial-init")
    print ("------------------------------------------")
    print ("Establishing connection to serial port:")
    try:
        #print ("TEST", baudrate,parity,bytesize,stopbits,timeout)
        ser = serial.Serial(port, baudrate=baudrate, parity=parity, bytesize=bytesize, stopbits=stopbits, timeout=timeout)
        print('.... Connection made.')
    except: 
        print('.... Connection flopped.')
        print('--- Aborting ---')
        sys.exit(2)
    print('')

    return ser


def lineread(ser,eol=None,timelimit=30):
    """
    DESCRIPTION:
       Does the same as readline(), but does not require a standard 
       linebreak character ('\r' in hex) to know when a line ends.
       Variable 'eol' determines the end-of-line char: '\x00'
       for the POS-1 magnetometer, '\r' for the envir. sensor.
       (Note: required for POS-1 because readline() cannot detect
       a linebreak and reads a never-ending line.)
    PARAMETERS:
       eol:   (string) lineend character(s): can be any kind of lineend
                           if not provided, then standard eol's are used
    """
    if not eol:
        eollist = ['\r','\x00','\n']
    else:
        eollist = [eol]

    timeout = time.time()+timelimit
    ser_str = ''
    while True:
        char = ser.read()
        #if char == '\x00':
        if char in eollist:
            break
        if time.time() > timeout:
            break
        ser_str += char
    return ser_str


def hexify_command(command,eol):
    """
    DESCRIPTION:
       This function translates the command text string into a hex
       string that the serial device can read. 'eol' is the 
       end-of-line character. '\r' for the environmental sensor,
       '\x00' for the POS-1 magnetometer.
    """
    commandstr = []
    for character in command:
        hexch = binascii.hexlify(character)
        commandstr.append(('\\x' + hexch).decode('string_escape'))

    command_hex = ''.join(commandstr) + (eol)

    return command_hex

"""
def send_command(ser,command,eol):
    command_hex = hexify_command(command,eol)
    ser.write(command_hex)
    response = lineread(ser,eol)
    print('Response: ', response)
"""

def send_command(ser,command,eol=None,hexify=False,bits=0,report=True):
    """
    DESCRIPTION:
        General method to send commands to a e.g. serial port.
        Returns the response
    PARAMETER:
        bits: (int) provide the amount of bits to be read
        eol:  (string) end-of-line string
    Options:
        POS1:           hexify=True, line=True, eol= '\x00'
        ENV05:          hexify=True, line=True, eol='\r'
        GSM90Sv6:       hexify=False, line=False, eol=''
        GSM90Sv7:       hexify=False, line=False, eol
        GSM90Fv7:       hexify=False, line=False, eol
    """
    if report:
        print('-- Sending command:  ', command)
    if hexify:
        command = hexify_command(command,eol)
        ser.write(command)
    else:
        if sys.version_info >= (3, 0):
            command = command.encode('ascii')
        if not eol:
            ser.write(command)
        else:
            ser.write(command+eol)
    if bits==0:
        response = lineread(ser,eol)
    else:
        response = ser.read(bits)
    if sys.version_info >= (3, 0):
        response = response.decode()
    if report:
        print('-- Response: ', response)
    return response

def datetime2array(t):
    return [t.year,t.month,t.day,t.hour,t.minute,t.second,t.microsecond]

def timeToArray(timestring):
    # Converts time string of format 2013-12-12T23:12:23.122324
    # to an array similiat to a datetime object
    try:
        splittedfull = timestring.split(' ')
        splittedday = splittedfull[0].split('-')
        splittedsec = splittedfull[1].split('.')
        splittedtime = splittedsec[0].split(':')
        datearray = splittedday + splittedtime
        datearray.append(splittedsec[1])
        datearray = list(map(int,datearray))
        return datearray
    except:
        print('Error while extracting time array')
        return []

def dataToFile(outputdir, sensorid, filedate, bindata, header):
    # File Operations
    try:
        #hostname = socket.gethostname()
        path = os.path.join(outputdir,sensorid)
        # outputdir defined in main options class
        if not os.path.exists(path):
            os.makedirs(path)
    except:
        print ("buffer {}: bufferdirectory could not be created - check permissions".format(sensorid))
    try:
        savefile = os.path.join(path, sensorid+'_'+filedate+".bin")
        if sys.version_info>(3,0,0):
            if not os.path.isfile(savefile):
                with open(savefile, "wb") as myfile:
                    head = "{}{}".format(header,"\n")
                    myfile.write(head.encode('utf-8'))
                    myfile.write(bindata)
                    myfile.write("\n".encode('utf-8'))
            else:
                with open(savefile, "ab") as myfile:
                    myfile.write(bindata)
                    myfile.write("\n".encode('utf-8'))
        else:
            if not os.path.isfile(savefile):
                with open(savefile, "wb") as myfile:
                    myfile.write(header + "\n")
                    myfile.write("{}{}".format(bindata,"\n"))
            else:
                with open(savefile, "a") as myfile:
                    myfile.write("{}{}".format(bindata,"\n"))
    except:
        print("buffer {}: Error while saving file".format(sensorid))


def dataToCSV(outputdir, sensorid, filedate, asciidata, header):
    """
    Writing buffer data to an ASCII file
    """
    # File Operations
    try:
        path = os.path.join(outputdir,sensorid)
        if not os.path.exists(path):
            os.makedirs(path)

        savefile = os.path.join(path, sensorid+'_'+filedate+".asc")
        asciilist = asciidata.split(';')
        if not os.path.isfile(savefile):
            with open(savefile, "wb") as csvfile:
                writer = csv.writer(csvfile,delimiter=';')
                writer.writerow(header)
                writer.writerow(asciilist)
        else:
            with open(savefile, "a") as csvfile:
                writer = csv.writer(csvfile,delimiter=';')
                writer.writerow(asciilist)
    except:
        print("buffer {}: Error while saving file".format(sensorid))


def AddSensor(path, dictionary, block=None):
    """
    DESCRIPTION:
        append sensor information to sensors.cfg
    PATH:
        sensors.conf
    """

    owheadline = "# OW block (automatically determined)"
    arduinoheadline = "# Arduino block (automatically determined)"
    mysqlheadline = "# SQL block (automatically determined)"
    owidentifier = '!'
    arduinoidentifier = '?'
    mysqlidentifier = '$'
    delimiter = '\n'

    def makeline(dictionary,delimiter):
        lst = []
        for el in SENSORELEMENTS:
            lst.append(str(dictionary.get(el,'-')))
        return ','.join(lst)+delimiter

    newline = makeline(dictionary,delimiter)

    # 1. if not block in ['OW','Arduino'] abort
    if not block in ['OW','ow','Ow','Arduino','arduino','ARDUINO','SQL','MySQL','mysql','MYSQL','sql']:
        print ("provided block needs to be 'OW', 'Arduino' or 'SQL'")
        return False

    # 2. check whether sensors.cfg existis
    # abort if not
    # read all lines
    try:
        sensors = open(path,'r')
    except:
        print ("could not read sensors.cfg")
        return False
    sensordata = sensors.readlines()
    if not len(sensordata) > 0:
        print ("no data found in sensors.cfg")
        return False

    if block in ['OW','ow','Ow']:
        num = [line for line in sensordata if line.startswith(owheadline)]
        identifier = owidentifier
        headline = owheadline
    elif block in ['Arduino','arduino','ARDUINO']:
        num = [line for line in sensordata if line.startswith(arduinoheadline)]
        identifier = arduinoidentifier
        headline = arduinoheadline
    elif block in ['SQL','MySQL','mysql','MYSQL','sql']:
        num = [line for line in sensordata if line.startswith(mysqlheadline)]
        identifier = mysqlidentifier
        headline = mysqlheadline

    # 3. Append/Insert line
    if len(num) > 0:
            cnt = [idx for idx,line in enumerate(sensordata) if line.startswith(identifier) or  line.startswith('#'+identifier)]
            lastline = max(cnt)
            if not (identifier+newline) in sensordata:
                if not ('#'+identifier+newline) in sensordata:
                    sensordata.insert(lastline+1,identifier+newline)
    else:
            sensordata.append(delimiter)
            sensordata.append(headline+delimiter)
            sensordata.append(identifier+newline)

    # 6. write all lines to sensors.cfg
    with open(path, 'w') as f:
        f.write(''.join(sensordata))

    return True


def GetSensors(path, identifier=None, secondidentifier=None):
    """
    DESCRIPTION:
        read sensor information from a file
        Now: just define them by hand
    PATH:
        sensors.conf
    CONTENT:
        # sensors.conf contains specific information for each attached sensor
        # ###################################################################
        # Information which need to be provided comprise:
        # sensorid: an unique identifier for the specfic sensor consiting of SensorName,
        #           its serial number and a revision number (e.g. GSM90_12345_0001)
        # connection: e.g. the port to which the instument is connected (USB0, S1, ACM0, DB-mydb) 
        # serial specifications: baudrate, bytesize, etc
        # sensor mode: passive (sensor is broadcasting data by itself)
        #              active (sensor is sending data upon request)
        # initialization info:
        #              None (passive sensor broadcasting data without initialization)
        #              [parameter] (passive sensor with initial init e.g. GSM90,POS1)
        #              [parameter] (active sensor, specific call parameters and wait time)
        # protocol:
        #
        # optional sensor description:
        #
        # each line contains the following information
        # sensorid port baudrate bytesize stopbits parity mode init protocol sensor_description
        #e.g. ENV05_2_0001;USB0;9600;8;1;EVEN;passive;None;Env;wic;A2;'Environment sensor measuring temperature and humidity'

        ENV05_2_0001;USB0;9600;8;1;EVEN;passive;None;Env;'Environment sensor measuring temperature and humidity'
    RETURNS:
        a dictionary containing:
        'sensorid':'ENV05_2_0001', 'port':'USB0', 'baudrate':9600, 'bytesize':8, 'stopbits':1, 'parity':'EVEN', 'mode':'a', 'init':None, 'rate':10, 'protocol':'Env', 'name':'ENV05', 'serialnumber':'2', 'revision':'0001', 'path':'-', 'pierid':'A2', 'ptime':'NTP', 'sensordesc':'Environment sensor measuring temperature and humidity'

    """
    sensors = open(path,'r')
    sensordata = sensors.readlines()
    sensorlist = []
    sensordict = {}
    elements = SENSORELEMENTS

    # add identifier here
    #

    for item in sensordata:
        sensordict = {}
        try:
            parts = item.split(',')
            if item.startswith('#'):
                continue
            elif item.isspace():
                continue
            elif item.startswith('!') and not identifier:
                continue
            elif item.startswith('?') and not identifier:
                continue
            elif item.startswith('$') and not identifier:
                continue
            elif not identifier and len(item) > 8:
                for idx,part in enumerate(parts):
                    sensordict[elements[idx]] = part
            elif item.startswith(str(identifier)) and len(item) > 8:
                if not secondidentifier:
                    for idx,part in enumerate(parts):
                        if idx == 0:
                            part = part.strip(str(identifier))
                        sensordict[elements[idx]] = part
                elif secondidentifier in parts:
                    for idx,part in enumerate(parts):
                        if idx == 0:
                            part = part.strip(str(identifier))
                        sensordict[elements[idx]] = part
                else:
                    pass
        except:
            # Possible issue - empty line
            pass
        if not sensordict == {}:
            sensorlist.append(sensordict)

    return sensorlist


def GetConf(path):
    """
    DESCRIPTION:
        read default configuration paths etc from a file
        Now: just define them by hand
    PATH:
        defaults are stored in magpymqtt.conf

        File looks like:
        # Configuration data for data transmission using MQTT (MagPy/MARTAS)
        # use # to uncomment
        # ##################################################################
        #
        # Working directory
        # -----------------
        # Please specify the path to the configuration files 
        configpath : /home/leon/CronScripts/MagPyAnalysis/MQTT

        # Definition of the bufferdirectory
        # ---------------------------------
        # Within this path, MagPy's write routine will store binary data files
        bufferdirectory : /srv/ws

        # Serial ports path
        # -----------------
        serialport : /dev/tty
        timeout : 60.0

        # MQTT definitions 
        # ----------------
        broker : localhost
        mqttport : 1883
        mqttdelay : 60

        # One wire configuration
        # ----------------------
        # ports: u for usb ---- NEW: owserver needs to be running - then just get port and address
        owport : usb
        owaddress : localhost

        # Logging
        # ----------------------
        # specify location to which logging information is send
        # e.g. sys.stdout , /home/cobs/logs/logmqtt.log
        logging : sys.stdout

    """
    # Init values:
    confdict = {}
    confdict['sensorsconf'] = '/home/leon/CronScripts/MagPyAnalysis/MQTT/sensors.cfg'
    #confdict['bufferdirectory'] = '/srv/ws'
    confdict['station'] = 'wic'
    confdict['bufferdirectory'] = '/srv/mqtt'
    confdict['serialport'] = '/dev/tty'
    confdict['timeout'] = 60.0
    confdict['broker'] = 'localhost'
    confdict['mqttport'] = 1883
    confdict['mqttdelay'] = 60
    confdict['logging'] = 'sys.stdout'
    confdict['owport'] = 4304
    confdict['owhost'] = 'localhost'

    try:
        config = open(path,'r')
        confs = config.readlines()

        for conf in confs:
            conflst = conf.split(':')
            if conf.startswith('#'): 
                continue
            elif conf.isspace():
                continue
            elif len(conflst) == 2:
                conflst = conf.split(':')
                key = conflst[0].strip()
                value = conflst[1].strip()
                confdict[key] = value
    except:
        print ("Problems when loading conf data from file. Using defaults")

    return confdict


def GetConf2(path, confdict={}):
    """
    Version 2020-10-28
    DESCRIPTION:
       can read a text configuration file and extract lists and dictionaries
    VARIBALES:
       path             Obvious
       confdict         provide default values
    SUPPORTED:
       key   :    stringvalue                                 # extracted as { key: str(value) }
       key   :    intvalue                                    # extracted as { key: int(value) }
       key   :    item1,item2,item3                           # extracted as { key: [item1,item2,item3] }
       key   :    subkey1:value1;subkey2:value2               # extracted as { key: {subkey1:value1,subkey2:value2} }
       key   :    subkey1:value1;subkey2:item1,item2,item3    # extracted as { key: {subkey1:value1,subkey2:[item1...]} }
    """
    exceptionlist = ['bot_id']

    def is_number(s):
        try:
            float(s)
            return True
        except ValueError:
            return False

    try:
        config = open(path,'r')
        confs = config.readlines()
        for conf in confs:
            conflst = conf.split(':')
            if conflst[0].strip() in exceptionlist or is_number(conflst[0].strip()):
                # define a list where : occurs in the value and is not a dictionary indicator
                conflst = conf.split(':',1)
            if conf.startswith('#'):
                continue
            elif conf.isspace():
                continue
            elif len(conflst) == 2:
                conflst = conf.split(':',1)
                key = conflst[0].strip()
                value = conflst[1].strip()
                # Lists
                if value.find(',') > -1:
                    value = value.split(',')
                    value = [el.strip() for el  in value]
                try:
                    confdict[key] = int(value)
                except:
                    confdict[key] = value
            elif len(conflst) > 2:
                # Dictionaries
                if conf.find(';') > -1 or len(conflst) == 3:
                    ele = conf.split(';')
                    main = ele[0].split(':')[0].strip()
                    cont = {}
                    for el in ele:
                        pair = el.split(':')
                        # Lists
                        subvalue = pair[-1].strip()
                        if subvalue.find(',') > -1:
                            subvalue = subvalue.split(',')
                            subvalue = [el.strip() for el  in subvalue]
                        try:
                            cont[pair[-2].strip()] = int(subvalue)
                        except:
                            cont[pair[-2].strip()] = subvalue
                    confdict[main] = cont
                else:
                    print ("Subdictionary expected - but no ; as element divider found")
    except:
        print ("Problems when loading conf data from file. Using defaults")

    return confdict





#!/usr/bin/env python
# coding=utf-8

"""
Testing notifications
"""

from martas.core.methods import martaslog as ml
import sys
import os
import getopt


# Basic MARTAS Telegram logging configuration for IMBOT manager


def main(argv):
    notificationtype = 'telegram'
    configpath = '/etc/martas/telegram.cfg'
    message = ''
    name = "TestMessage"
    logpath = '/var/log/magpy/testmessage.log'
    note = {}
    proxies = {}
    debug = False

    try:
        opts, args = getopt.getopt(argv,"hn:m:c:l:p:D",["notificationtype=","message=","logname=","logpath=","debug=",])
    except getopt.GetoptError:
        print ('testnote.py -n <notificationtype> -m <message> -c <configpath> -l <logname> -p <logpath>')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print ('-------------------------------------')
            print ('Description:')
            print ('-- testnote.py will send a note to the specificied notification channel --')
            print ('-----------------------------------------------------------------')
            print ('testnote is a python3 program to send')
            print ('a note.')
            print ('')
            print ('-------------------------------------')
            print ('Usage:')
            print ('python3 testnote.py -n <notificationtype> -m <message>')
            print ('-------------------------------------')
            print ('Options:')
            print ('-m            : path for telegram configuration file')
            print ('-n            : email, telegram or log')
            print ('-c            : configuration path')
            print ('-l            : logname')
            print ('-p            : logpath')
            print ('-------------------------------------')
            print ('Example of memory:')
            print ('-------------------------------------')
            print ('Application:')
            print ('python3 testnote.py -n email -m "Hello World" -c /etc/martas/mail.cfg -l TestMessage -p /home/user/test.log')
            print ('python3 testnote.py -n telegram -m "Hello World, I am here" -c /etc/martas/telegram.cfg -l TestMessage -p /home/leon/test.log')
            print ('python3 testnote.py -n log -m "Hello World again" -l TestMessage -p /home/leon/test.log')
            print ('python3 testnote.py -n telegram -m "Hello World" -c ~/.martas/conf/telegram.cfg -l TestMessage -p /home/leon/test.log -o http://192.168.1.1:3128')
            sys.exit()
        elif opt in ("-n", "--notifictaiontype"):
            notificationtype = arg
        elif opt in ("-m", "--message"):
            message = arg
        elif opt in ("-c", "--configpath"):
            configpath = os.path.abspath(arg)
        elif opt in ("-l", "--logname"):
            name = arg
        elif opt in ("-p", "--logpath"):
            logpath = os.path.abspath(arg)
        elif opt in ("-o", "--proxy"):
            proxies = {'https', arg}
        elif opt in ("-D", "--debug"):
            debug = True

    if debug:
        print ("Basic code test OK")
        sys.exit(0)
        
    if not notificationtype in ['email','telegram','log']:
        print ("Notification type not supported. Needs to be one of email, telegram, log")
        sys.exit()

    note[name] = message

    martaslog = ml(logfile=logpath,receiver=notificationtype, proxies=proxies)
    martaslog.telegram['config'] = configpath
    martaslog.email['config'] = configpath
    martaslog.msg(note)


if __name__ == "__main__":
   main(sys.argv[1:])





#!/usr/bin/env python
# coding=utf-8

"""
MagPy - telegram bot interaction
################################

Package requirements:
pip install geomagpy (>= 0.3.99)
pip install telepot (>= )

Optional python packages:
pip install psutil    # status memory, cpu
pip install platform  # system definitions

Optional linux packages
sudo apt-get install fswebcam   # getting cam pictures

Optional for remote ssh link opening:
install tmate version >= 2.4

add cronstop to regularly restart the bot (root)
sudo crontab -e
PATH=/bin/sh
0  6  *  *  *  /etc/init.d/telegrambot restart > /dev/NULL 2&>1
0  14  *  *  *  /etc/init.d/telegrambot restart > /dev/NULL 2&>1
0  22  *  *  *  /etc/init.d/telegrambot restart > /dev/NULL 2&>1


# ADD Option to locate configuration file

Tool for interaction with remote systems:
Commands: external stations: - status, getlog (amount of lines), martas (restart martas), healthstate (disk status, processor), type

Commands: cobs:              - checkDB, get nagios infi

"""

from martas.core.message import ActionHandler

# Define packages to be used (local refers to test environment)
# ------------------------------------------------------------
import telepot
from telepot.loop import MessageLoop
import logging
import os
import urllib3
import time


def setuplogger(name='telegrambot',loglevel='DEBUG',path='stdout'):

    logpath = None
    try:
        level = eval("logging.{}".format(loglevel))
    except:
        level = logging.DEBUG
    if not path in ['sys.stdout','stdout']:
        logpath = path
    # create formatter
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s : %(message)s',
                              "%Y-%m-%d %H:%M:%S")

    logger = logging.getLogger(name)
    logger.setLevel(level)
    if logpath:
        print ("telegrambot: Creating log file")
        # create file handler which logs even debug messages
        fh = logging.FileHandler(logpath)
        fh.setLevel(level)
        # create console handler with a higher log level
        ch = logging.StreamHandler()
        ch.setLevel(logging.ERROR)
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)
        logger.addHandler(fh)
        logger.addHandler(ch)
    else:
        print ("telegrambot: logging to stdout")
        ch = logging.StreamHandler()
        ch.setLevel(level)
        ch.setFormatter(formatter)
        logger.addHandler(ch)

    return logger


mainpath = os.path.dirname(os.path.realpath(__file__))
telegramcfgpath = os.path.join(mainpath,"..","conf","telegrambot.cfg")
ah = ActionHandler(configpath=telegramcfgpath, commands=None)

tglogger = setuplogger(name='telegrambot',loglevel=ah.configuration.get('loglevel'),path=ah.configuration.get('bot_logging').strip())

proxy = ah.configuration.get('proxy','').strip()
proxyport = ah.configuration.get('proxyport','').strip()

if proxy:
    print (" found proxy")
    proxy_url="http://{}:{}".format(proxy,proxyport)
    telepot.api._pools = {'default': urllib3.ProxyManager(proxy_url=proxy_url, num_pools=3, maxsize=10, retries=False, timeout=30),}
    telepot.api._onetime_pool_spec = (urllib3.ProxyManager, dict(proxy_url=proxy_url, num_pools=1, maxsize=1, retries=False, timeout=30))
    print (" ... established to {}".format(proxy_url))

    # Extract command lists


def handle(msg):
    content_type, chat_type, chat_id = telepot.glance(msg)
    tglogger.info("Bot -> ContentType: {}; ChatType: {}".format(content_type, chat_type))
    firstname = msg['from']['first_name']
    userid = msg['from']['id']
    ah.messageconfig['firstname'] = firstname

    chat_id = msg['chat']['id']
    command = msg['text'].replace('/','')

    if not str(chat_id) in ah.configuration.get('allowed_users'):
        bot.sendMessage(chat_id, "My mother told me not to speak to strangers, sorry...")
        tglogger.warning('--------------------- Unauthorized access -------------------------')
        tglogger.warning('!!! unauthorized access attempt ({}) from ChatID {} (User: {}) !!!'.format(command,chat_id,firstname))
        tglogger.warning('-------------------------------------------------------------------')
    else:
        if content_type == 'text':
            tglogger.info('Received command "{}" from ChatID {} (User: {})'.format(command,chat_id,firstname))
            al = ah.interpret(command)
            message = ah.execute_action(al, input=command, debug=True)
            for com in message:
                cc = message.get(com)
                comcontent = cc.get('message')
                text = comcontent.get("text","")
                calls = comcontent.get("call",[])
                pics = comcontent.get("pictures",[])
                for call in calls:
                    print ("executing call", call)
                    text += ah.execute_call(call)
                    if 'fswebcam' in call:
                        pics.append('/tmp/webimage.jpg')
                if text:
                    bot.sendMessage(chat_id, text, parse_mode='Markdown')
                for pic in pics:
                    if os.path.isfile(pic):
                        bot.sendPhoto(chat_id, open(pic,'rb'))

bot = telepot.Bot(ah.configuration.get('bot_id'))
MessageLoop(bot, handle).run_as_thread()
tglogger.info('Listening ...')

# Keep the program running.
while 1:
    try:
        time.sleep(10)
    except KeyboardInterrupt:
        tglogger.info('\n Program interrupted')
        exit()
    except:
        tglogger.error('Other error or exception occurred!')
